/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.client;

import java.io.File;
import java.net.URL;
import java.security.cert.X509Certificate;

import kgrid.raems.service.impl.Consts;

import org.apache.axis.message.addressing.EndpointReferenceType;
import org.globus.axis.util.Util;
import org.globus.delegation.DelegationUtil;
import org.globus.exec.client.GramJob;
import org.globus.exec.generated.JobDescriptionType;
import org.globus.exec.generated.ManagedJobFactoryPortType;
import org.globus.exec.utils.ManagedJobFactoryConstants;
import org.globus.exec.utils.client.ManagedJobFactoryClientHelper;
import org.globus.exec.utils.rsl.RSLHelper;
import org.globus.gsi.GSIConstants;
import org.globus.gsi.GlobusCredential;
import org.globus.wsrf.impl.security.authorization.Authorization;
import org.globus.wsrf.impl.security.authorization.HostAuthorization;
import org.globus.wsrf.impl.security.descriptor.ClientSecurityDescriptor;
import org.globus.wsrf.utils.AddressingUtils;
import org.gridforum.jgss.ExtendedGSSManager;
import org.ietf.jgss.GSSCredential;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 */
public class RAEMS_Client_2 implements Consts {
    static {
        Util.registerTransport();
    }

    public static void main(String[] args) {

        try {
            // String hostName = "cassandra.cs.icar.cnr.it";
            String hostName = "192.168.1.39";
            String rsl_Input_File = "./abstract model/abstract_model0.xml";

            URL factoryUrl = ManagedJobFactoryClientHelper.getServiceURL(
                    "https://" + hostName + ":" + CONTAINER_PORT + "/"
                            + GRAM_FACTORY).getURL();
            String factoryType = ManagedJobFactoryConstants.FACTORY_TYPE.FORK;
            EndpointReferenceType factoryEndpoint = ManagedJobFactoryClientHelper
                    .getFactoryEndpoint(factoryUrl, factoryType);
            ManagedJobFactoryPortType factoryPort = ManagedJobFactoryClientHelper
                    .getPort(factoryEndpoint);

            File jobDescriptionFile = new File(rsl_Input_File);
            JobDescriptionType jobDescription = RSLHelper
                    .readRSL(jobDescriptionFile);
            GramJob job = new GramJob(jobDescription);

            ExtendedGSSManager manager = (ExtendedGSSManager) ExtendedGSSManager
                    .getInstance();
            GSSCredential cred = manager
                    .createCredential(GSSCredential.INITIATE_AND_ACCEPT);

            job.setCredentials(cred);

            job.submit(factoryEndpoint);
            // job.addListener(this);

            System.out.println("Handler Complete.");
            // TO DO: Create functional listener, makes the server send it's
            // state to the client
            boolean control = false;
            while (!control) {

                if (job.getState() != null
                        && ((job.getState().toString()).equals("Done") || (job
                                .getState().toString()).equals("Failed")))
                    control = true;

                // this.sleep(900);
            }

        } catch (Exception e) {
            // raemsResource.setModel("STATE:"+rsl_Input_File+"-FAILED");
            System.out.println("Exception thrown: " + e.toString());
            e.printStackTrace();
        }

    }// main

    private static EndpointReferenceType delegateCredential(String host,
            String port) throws Exception {
        ClientSecurityDescriptor desc = getClientSecDesc();
        GlobusCredential credential = GlobusCredential.getDefaultCredential();
        String factoryUrl = ("https://" + host + ":" + port + "/" + DELEGATION_FACTORY);
        int terminationTime = 60;
        int lifetime = terminationTime * 60; // lifetime in seconds
        EndpointReferenceType delegEpr = AddressingUtils
                .createEndpointReference(factoryUrl, null);
        X509Certificate[] certsToDelegateOn = DelegationUtil
                .getCertificateChainRP(delegEpr, desc);
        X509Certificate certToSign = certsToDelegateOn[0];
        return DelegationUtil.delegate(factoryUrl, credential, certToSign,
                lifetime, false, desc);
    }

    private static ClientSecurityDescriptor getClientSecDesc() {
        ClientSecurityDescriptor desc = new ClientSecurityDescriptor();
        Authorization authzVal = HostAuthorization.getInstance();
        Object authVal = GSIConstants.SIGNATURE;
        desc.setGSITransport((Integer) authVal);
        Util.registerTransport();
        desc.setAuthz(authzVal);
        return desc;
    }
}
