/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.client;

import java.net.InetAddress;
import java.security.cert.X509Certificate;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import kgrid.raems.service.impl.Consts;
import kgrid.raems.stubs.RAEMSFactory.CreateResource;
import kgrid.raems.stubs.RAEMSFactory.CreateResourceResponse;
import kgrid.raems.stubs.RAEMSFactory.RAEMSFactoryPortType;
import kgrid.raems.stubs.RAEMSFactory.service.RAEMSFactoryServiceAddressingLocator;
import kgrid.raems.stubs.RAEMSService.ExecutionParameters;
import kgrid.raems.stubs.RAEMSService.RAEMSPortType;
import kgrid.raems.stubs.RAEMSService.service.RAEMSServiceAddressingLocator;

import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.globus.axis.util.Util;
import org.globus.delegation.DelegationUtil;
import org.globus.gsi.GSIConstants;
import org.globus.gsi.GlobusCredential;
import org.globus.wsrf.impl.security.authorization.Authorization;
import org.globus.wsrf.impl.security.authorization.HostAuthorization;
import org.globus.wsrf.impl.security.descriptor.ClientSecurityDescriptor;
import org.globus.wsrf.utils.AddressingUtils;
import org.w3c.dom.Document;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 */
public class RAEMS_Client_OLD implements Consts {
	static {
		Util.registerTransport();
	}

	public static void main(String[] args) {

		RAEMSFactoryServiceAddressingLocator factoryLocator = new RAEMSFactoryServiceAddressingLocator();
		RAEMSServiceAddressingLocator instanceLocator = new RAEMSServiceAddressingLocator();

		try {

			String factoryURI = args[0];
			EndpointReferenceType factoryEPR, instanceEPR;
			RAEMSFactoryPortType raemsFactory;
			RAEMSPortType raems;
			System.out.println("Get Factory Port Type");
			// 1: get Factory Port Type
			factoryEPR = new EndpointReferenceType();
			factoryEPR.setAddress(new Address(factoryURI));
			raemsFactory = factoryLocator
					.getRAEMSFactoryPortTypePort(factoryEPR);
			System.out
					.println("Create resource and get endpoint reference of WS-Resource");
			// 2: Create resource and get endpoint reference of WS-Resource.
			// This resource is our "instance".

			CreateResourceResponse createResponse = raemsFactory
					.createResource(new CreateResource());
			instanceEPR = createResponse.getEndpointReference();

			// 3
			System.out.println("Invocation method");
			raems = instanceLocator.getRAEMSPortTypePort(instanceEPR);
			EndpointReferenceType epr = delegateCredential(InetAddress
					.getLocalHost().getHostName(), CONTAINER_PORT);
			System.out.println("epr=" + epr.toString());
			raems = instanceLocator.getRAEMSPortTypePort(instanceEPR);
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder parser = factory.newDocumentBuilder();
			Document d = parser.parse("./abstract model/abstract_model0.xml");
			// "/home/marino/Desktop/TEST/abstract_model0.xml");
			d.normalize();

			ExecutionParameters ep = new ExecutionParameters(epr, d);
			System.out.println(d.getChildNodes().item(0).getNodeName());
			// ep.setExecutionPlanDoc(new DataHandler(new
			// FileDataSource("/home/globus/TEST/abstract_model0.xml")));
			// ep.setEprCredential(epr);
			raems.manageKExecution(ep);
			System.out.println("Start");

		} catch (Exception e) {
			e.printStackTrace();
		}

	}// main

	private static EndpointReferenceType delegateCredential(String host,
			int port) throws Exception {
		ClientSecurityDescriptor desc = getClientSecDesc();
		GlobusCredential credential = GlobusCredential.getDefaultCredential();
		String factoryUrl = ("https://" + host + ":" + port + "/" + Consts.DELEGATION_FACTORY);
		int terminationTime = 60;
		int lifetime = terminationTime * 60; // lifetime in seconds
		EndpointReferenceType delegEpr = AddressingUtils
				.createEndpointReference(factoryUrl, null);
		X509Certificate[] certsToDelegateOn = DelegationUtil
				.getCertificateChainRP(delegEpr, desc);
		X509Certificate certToSign = certsToDelegateOn[0];
		return DelegationUtil.delegate(factoryUrl, credential, certToSign,
				lifetime, false, desc);
	}

	private static ClientSecurityDescriptor getClientSecDesc() {
		ClientSecurityDescriptor desc = new ClientSecurityDescriptor();
		Authorization authzVal = HostAuthorization.getInstance();
		Object authVal = GSIConstants.SIGNATURE;
		desc.setGSITransport((Integer) authVal);
		Util.registerTransport();
		desc.setAuthz(authzVal);
		return desc;
	}
}
