/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.client;

import java.net.InetAddress;
import java.security.cert.X509Certificate;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import kgrid.raems.service.impl.Consts;
import kgrid.raems.stubs.RAEMSFactory.CreateResource;
import kgrid.raems.stubs.RAEMSFactory.CreateResourceResponse;
import kgrid.raems.stubs.RAEMSFactory.RAEMSFactoryPortType;
import kgrid.raems.stubs.RAEMSFactory.service.RAEMSFactoryServiceAddressingLocator;
import kgrid.raems.stubs.RAEMSService.ExecutionParameters;
import kgrid.raems.stubs.RAEMSService.RAEMSPortType;
import kgrid.raems.stubs.RAEMSService.service.RAEMSServiceAddressingLocator;

import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.globus.axis.util.Util;
import org.globus.delegation.DelegationUtil;
import org.globus.gsi.GSIConstants;
import org.globus.gsi.GlobusCredential;
import org.globus.wsrf.NotificationConsumerManager;
import org.globus.wsrf.NotifyCallback;
import org.globus.wsrf.WSNConstants;
import org.globus.wsrf.core.notification.ResourcePropertyValueChangeNotificationElementType;
import org.globus.wsrf.impl.security.authorization.Authorization;
import org.globus.wsrf.impl.security.authorization.HostAuthorization;
import org.globus.wsrf.impl.security.descriptor.ClientSecurityDescriptor;
import org.globus.wsrf.utils.AddressingUtils;
import org.oasis.wsn.Subscribe;
import org.oasis.wsn.TopicExpressionType;
import org.oasis.wsrf.properties.ResourcePropertyValueChangeNotificationType;
import org.w3c.dom.Document;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 */
public class RaemsModelListener implements NotifyCallback, Consts {

	static {
		Util.registerTransport();
	}

	private static String servizio = "";

	public synchronized void deliver(List topicPath,
			EndpointReferenceType producer, Object message) {
		synchronized (this) {
			ResourcePropertyValueChangeNotificationElementType notif_elem;
			ResourcePropertyValueChangeNotificationType notif;

			notif_elem = (ResourcePropertyValueChangeNotificationElementType) message;
			notif = notif_elem.getResourcePropertyValueChangeNotification();

			if (notif != null) {
				for (MessageElement me : notif.getNewValue().get_any()) {
					System.out.println(me.getValue());
				}
			}

			this.notify();
		}

	}

	public static RAEMSPortType getRAEMSPortType() throws Exception {
		RAEMSFactoryServiceAddressingLocator factoryLocator = new RAEMSFactoryServiceAddressingLocator();
		RAEMSServiceAddressingLocator instanceLocator = new RAEMSServiceAddressingLocator();

		try {

			String factoryURI = servizio;
			EndpointReferenceType factoryEPR, instanceEPR;
			RAEMSFactoryPortType raemsFactory;
			RAEMSPortType raems;
			System.out.println("Get Factory Port Type");
			// 1: get Factory Port Type
			factoryEPR = new EndpointReferenceType();
			factoryEPR.setAddress(new Address(factoryURI));
			raemsFactory = factoryLocator
					.getRAEMSFactoryPortTypePort(factoryEPR);
			System.out
					.println("Create resource and get endpoint reference of WS-Resource");

			// 2: Create resource and get endpoint reference of WS-Resource.
			// This resource is our "instance".
			CreateResource cr = new CreateResource();
			CreateResourceResponse createResponse = raemsFactory
					.createResource(cr);
			instanceEPR = createResponse.getEndpointReference();

			// 3
			System.out.println("Invocation method");
			raems = instanceLocator.getRAEMSPortTypePort(instanceEPR);
			return raems;
		} catch (Exception e) {
		}
		return null;

	}// getKDSPortType

	/**
	 * Subscribe the specified listener as consumer of notification 'KDSModel'
	 * topic of the specified KDS Service
	 * 
	 * @param listener
	 *            consumer of the notification
	 * @param kds
	 *            KDSPortType of the KDS Service whose the topic is a resource
	 *            property
	 */
	public static void subscribeToRAEMSModelTopic(RaemsModelListener listener,
			RAEMSPortType raems) throws Exception {

		// 1: set up the notification consumer
		// (the listener will be set up as a notification consumer)
		NotificationConsumerManager consumer = null;

		// 2: set the 'listener' as a 'notification consumer'
		consumer = NotificationConsumerManager.getInstance();
		consumer.startListening();
		EndpointReferenceType consumerEPR = consumer
				.createNotificationConsumer(listener);

		// 3: create (and opportunetely set) the remote subscribe request
		// Create the request to the remote Subscribe() call
		Subscribe request = new Subscribe();
		// Must the notification be delivered using the Notify
		// operation?
		request.setUseNotify(Boolean.TRUE);
		// Indicate what the consumer client's EPR is
		request.setConsumerReference(consumerEPR);

		// 4: create the topic to be subscribed to
		// The TopicExpression specifies what topic we want to
		// subscribe to
		TopicExpressionType topicExpression = new TopicExpressionType();
		topicExpression.setDialect(WSNConstants.SIMPLE_TOPIC_DIALECT);
		topicExpression.setValue(RP_MODEL);
		request.setTopicExpression(topicExpression);

		// 6: send the subscribe request to the notification producer (KDS)
		// IMPORTANT: the consumer DASListener is subscribed to the 'KDS_Model'
		// topic
		raems.subscribe(request);
	}// subscribe

	private static EndpointReferenceType delegateCredential(String host,
			int port) throws Exception {
		ClientSecurityDescriptor desc = getClientSecDesc();
		GlobusCredential credential = GlobusCredential.getDefaultCredential();
		String factoryUrl = ("https://" + host + ":" + port + "/" + DELEGATION_FACTORY);
		int terminationTime = 60;
		int lifetime = terminationTime * 60; // lifetime in seconds
		EndpointReferenceType delegEpr = AddressingUtils
				.createEndpointReference(factoryUrl, null);
		X509Certificate[] certsToDelegateOn = DelegationUtil
				.getCertificateChainRP(delegEpr, desc);
		X509Certificate certToSign = certsToDelegateOn[0];
		return DelegationUtil.delegate(factoryUrl, credential, certToSign,
				lifetime, false, desc);
	}

	private static ClientSecurityDescriptor getClientSecDesc() {
		ClientSecurityDescriptor desc = new ClientSecurityDescriptor();
		Authorization authzVal = HostAuthorization.getInstance();
		Object authVal = GSIConstants.SIGNATURE;
		desc.setGSITransport((Integer) authVal);
		Util.registerTransport();
		desc.setAuthz(authzVal);
		return desc;
	}

	public static void main(String[] args) {
		RaemsModelListener rml = new RaemsModelListener();
		String xml_path = "";
		try {
			if (args.length != 2) {
				System.out.println("usage");
				System.out.println("ip, abstractXmlPath");
				return;

			} else {

				xml_path = args[1];
				servizio = "https://" + args[0] + ":" + CONTAINER_PORT + "/"
						+ RAEMS_FACTORY;
			}
			RAEMSPortType portType = getRAEMSPortType();
			subscribeToRAEMSModelTopic(rml, portType);
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder parser = factory.newDocumentBuilder();
			Document d =
			// parser.parse("./abstractmodel/abstract_model_consecutive_test.xml");
			parser.parse(xml_path);
			d.normalize();
			EndpointReferenceType epr = delegateCredential(InetAddress
					.getLocalHost().getHostAddress(), CONTAINER_PORT);
			ExecutionParameters ep = new ExecutionParameters(epr, d);

			System.out.println("Listening to notifications...");
			portType.manageKExecution(ep);
			while (true) {
				synchronized (rml) {
					rml.wait();

				}
			}
		} catch (Throwable e1) {
			e1.printStackTrace();
		}

	}
}
