/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.gui;

import java.util.Iterator;
import java.util.Observer;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import edu.uci.ics.jung.graph.DirectedSparseGraph;
import edu.uci.ics.jung.graph.util.EdgeType;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 */
public class ExecutionPlanGraph extends DirectedSparseGraph<Task, String> {

    private static final long serialVersionUID = -8777236747758052674L;

    public void init(Document d) {
        NodeList taskList = d.getElementsByTagName("Task");
        for (int i = 0; i < taskList.getLength(); i++) {
            Node n = taskList.item(i);
            NamedNodeMap attrs = n.getAttributes();
            Node label = attrs.getNamedItem("ep:label");
            Node name = attrs.getNamedItem("name");
            Task t = null;
            if (name != null) {
                t = new Task(name.getTextContent());
            } else if (label != null) {
                t = new Task(label.getTextContent());
            }
            if (n.getFirstChild() == null)
                t.setType(TaskType.DUMMY);
            else if (getFirstNonTextChild(n).getNodeName()
                    .equals("Computation"))
                t.setType(TaskType.COMPUTATION);
            else if (getFirstNonTextChild(n).getNodeName().equals(
                    "DataTransfer"))
                t.setType(TaskType.DATA_TRANSFER);
            this.addVertex(t);
        }
        NodeList nl = d.getElementsByTagName("TaskLink");
        for (int i = 0; i < nl.getLength(); i++) {
            Node n = nl.item(i);
            NamedNodeMap attrs = n.getAttributes();
            Task t1 = getTask(attrs.getNamedItem("ep:from").getTextContent());
            Task t2 = getTask(attrs.getNamedItem("ep:to").getTextContent());
            this.addEdge(t1.getName() + "-" + t2.getName(), t1, t2,
                    EdgeType.DIRECTED);
        }
    }

    private Node getFirstNonTextChild(Node n) {
        NodeList children = n.getChildNodes();
        for (int i = 0; i < children.getLength(); i++) {
            Node child = children.item(i);
            if (child.getNodeType() != Node.TEXT_NODE)
                return child;
        }
        return null;
    }

    public void setTaskStatus(String taskname, TaskState status) {
        Iterator it = this.getVertices().iterator();
        while (it.hasNext()) {
            Task t = (Task) it.next();
            if (t.getName().equals(taskname))
                t.setStatus(status);
        }
    }

    public Task getTask(String taskName) {
        Iterator it = this.getVertices().iterator();
        while (it.hasNext()) {
            Task t = (Task) it.next();
            if (t.getName().equals(taskName))
                return t;
        }
        return null;
    }

    public void subcribeObserver(Observer o) {
        for (Task t : this.getVertices())
            t.addObserver(o);
    }

}
