/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.util.HashMap;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JPanel;

import org.apache.commons.collections15.Transformer;

import edu.uci.ics.jung.algorithms.layout.FRLayout;
import edu.uci.ics.jung.algorithms.layout.Layout;
import edu.uci.ics.jung.visualization.GraphZoomScrollPane;
import edu.uci.ics.jung.visualization.LayeredIcon;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.control.GraphMousePlugin;
import edu.uci.ics.jung.visualization.control.LayoutScalingControl;
import edu.uci.ics.jung.visualization.control.PickingGraphMousePlugin;
import edu.uci.ics.jung.visualization.control.PluggableGraphMouse;
import edu.uci.ics.jung.visualization.decorators.DefaultVertexIconTransformer;
import edu.uci.ics.jung.visualization.decorators.EllipseVertexShapeTransformer;
import edu.uci.ics.jung.visualization.decorators.ToStringLabeller;
import edu.uci.ics.jung.visualization.decorators.VertexIconShapeTransformer;
import edu.uci.ics.jung.visualization.renderers.BasicVertexLabelRenderer;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 */
public class ExecutionPlanVisualizationViewer implements Observer{
	JPanel pane=null;
//	JPanel pane;
	VisualizationViewer<Task, String> viewer;
	Layout<Task, String> layout;
	PluggableGraphMouse graphMouse;
	Map<Task,Icon> iconMap=new HashMap<Task,Icon>();
	VertexIconShapeTransformer<Task> iconShapetransformer;
	DefaultVertexIconTransformer<Task> iconTransformer;

	public ExecutionPlanVisualizationViewer() throws InterruptedException {
		layout=new FRLayout<Task, String>(new ExecutionPlanGraph());
		layout.setSize(new Dimension(450, 450));
		viewer = new VisualizationViewer<Task, String>(layout,new Dimension(500,500));
		viewer.setGraphMouse(graphMouse);
		viewer.setBackground(Color.white);
		viewer.getRenderContext().setVertexShapeTransformer(getVertexShapeTransformer());
		viewer.getRenderContext().setVertexIconTransformer(getVertexIconTransformer());
		viewer.getRenderContext().setVertexLabelTransformer(
				new ToStringLabeller<Task>());
		graphMouse = new PluggableGraphMouse();
		graphMouse.add(new PickingGraphMousePlugin<Task, String>());
		viewer.getRenderer().setVertexLabelRenderer(new BasicVertexLabelRenderer());
		viewer.scaleToLayout(new LayoutScalingControl());
		pane = new GraphZoomScrollPane(viewer);
		pane.setPreferredSize(new Dimension(500,500));
	}

	private Transformer<Task, Icon> getVertexIconTransformer() {
		if(iconTransformer!=null)
			return iconTransformer;
		iconTransformer=new DefaultVertexIconTransformer<Task>();
//		Map<Task,Icon> iconMap=new Hashtable<Task,Icon>();
		iconTransformer.setIconMap(iconMap);
		return iconTransformer;
	}

	private Transformer getVertexShapeTransformer() {
		if(iconShapetransformer!=null)
			return iconShapetransformer;
		iconShapetransformer=new VertexIconShapeTransformer<Task>(new EllipseVertexShapeTransformer<Task>());
		Icon i = 
            new LayeredIcon(new ImageIcon("./img/task-48.png").getImage());
		for(Task t:layout.getGraph().getVertices()){
			iconMap.put(t, i);
		}
		iconShapetransformer.setIconMap(iconMap);
		return iconShapetransformer;
	}

	public Component getComponent() {
		return pane;
	}

	public void fireTaskStateChanged(Task t){
		iconMap.put(t, IconCache.getTaskIcon(t));
	}
	public void repaintChanged() {
		 viewer.repaint(80);
	}

	public void setGraph(ExecutionPlanGraph graph) {
		layout=new FRLayout<Task, String>(graph);
		Dimension d=(pane.getSize());
		layout.setSize(new Dimension((int)(d.getWidth()*1.3),(int)(d.getHeight()*1.3)));
		layout.setGraph(graph);
		viewer.setGraphLayout(layout);
		for(Task t:graph.getVertices()){
			iconMap.put(t, IconCache.getTaskIcon(t));
		}
		viewer.getRenderContext().setVertexShapeTransformer(getVertexShapeTransformer());
		viewer.getRenderContext().setVertexIconTransformer(getVertexIconTransformer());
		viewer.scaleToLayout(new LayoutScalingControl());
		graph.subcribeObserver(this);
		repaintChanged();
	}

	public ExecutionPlanGraph getGraph() {
		return (ExecutionPlanGraph) layout.getGraph();
	}
	
	public void addGraphMousePlugin(GraphMousePlugin plugin){
		graphMouse.add(plugin);
		viewer.setGraphMouse(graphMouse);
	}

	/**
	 * 
	 */
	private static final long serialVersionUID = -2113468302381715346L;
	/** 
     * this class exists only to provide settings to turn on/off shapes and image fill
     * in this demo.
     * In a real application, use VertexIconShapeTransformer instead.
     * 
     */

	@Override
	public void update(Observable o, Object arg) {
		if(o instanceof Task){
			Task t=(Task)o;
			iconMap.put(t, IconCache.getTaskIcon(t));
			
			this.getVertexIconTransformer().transform(t);
			this.repaintChanged();
		}
		
	}
   
}
