/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.gui;

import java.awt.Component;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.util.HashMap;

import javax.swing.Icon;
import javax.swing.ImageIcon;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 */
public class IconCache {
    private static HashMap<TaskIconIdentifier, Icon> icons = new HashMap<TaskIconIdentifier, Icon>();

    public static Icon getTaskIcon(Task task) {
        // System.out.println(icons);
        Icon i = icons.get(new TaskIconIdentifier(task.getType(), task
                .getStatus()));
        return i;
    }

    public static void buildIconCache(Component c) throws InterruptedException {
        ImageIcon baseComputationIcon = new ImageIcon("./img/task-48.png");
        ImageIcon baseDataTransferIcon = new ImageIcon(
                "./img/datatransfer-48.png");
        ImageIcon baseDummyIcon = new ImageIcon("./img/dummy-24.png");
        // _----------------------------------------_//
        MediaTracker tracker = new MediaTracker(c);
        Image unavailableImage = Toolkit.getDefaultToolkit().getImage(
                "./img/unavailable.png");
        tracker.addImage(unavailableImage, 0);
        Image createdImage = Toolkit.getDefaultToolkit().getImage(
                "./img/created.png");
        tracker.addImage(createdImage, 0);
        Image runningImage = Toolkit.getDefaultToolkit().getImage(
                "./img/running.png");
        tracker.addImage(runningImage, 0);
        Image doneImage = Toolkit.getDefaultToolkit()
                .getImage("./img/done.png");
        tracker.addImage(doneImage, 0);
        Image failedImage = Toolkit.getDefaultToolkit().getImage(
                "./img/failed.png");
        tracker.addImage(failedImage, 0);
        Image destroyed_failedImage = Toolkit.getDefaultToolkit().getImage(
                "./img/destroyed-failed.png");
        tracker.addImage(destroyed_failedImage, 0);
        Image destroyed_doneImage = Toolkit.getDefaultToolkit().getImage(
                "./img/destroyed-done.png");
        tracker.addImage(destroyed_doneImage, 0);
        tracker.waitForAll();

        // _----------------------------------------_//
        ImageIcon unavailableOverlayIcon = new ImageIcon(unavailableImage);
        ImageIcon createdOverlayIcon = new ImageIcon(createdImage);
        ImageIcon runningOverlayIcon = new ImageIcon(runningImage);
        ImageIcon doneOverlayIcon = new ImageIcon(doneImage);
        ImageIcon failedOverlayIcon = new ImageIcon(failedImage);
        ImageIcon destroyed_failedOverlayIcon = new ImageIcon(
                destroyed_failedImage);
        ImageIcon destroyed_doneOverlayIcon = new ImageIcon(destroyed_doneImage);
        ImageIcon destroyed_unkOverlayIcon = new ImageIcon(unavailableImage);
        // _----------------------------------------_//
        ImageIcon c_unavailableIcon = createOverlayedIcon(baseComputationIcon,
                unavailableOverlayIcon);
        ImageIcon c_createdIcon = createOverlayedIcon(baseComputationIcon,
                createdOverlayIcon);
        ImageIcon c_runningIcon = createOverlayedIcon(baseComputationIcon,
                runningOverlayIcon);
        ImageIcon c_doneIcon = createOverlayedIcon(baseComputationIcon,
                doneOverlayIcon);
        ImageIcon c_failedIcon = createOverlayedIcon(baseComputationIcon,
                failedOverlayIcon);
        ImageIcon c_destroyed_failedIcon = createOverlayedIcon(
                baseComputationIcon, destroyed_failedOverlayIcon);
        ImageIcon c_destroyed_doneIcon = createOverlayedIcon(
                baseComputationIcon, destroyed_doneOverlayIcon);
        ImageIcon c_destroyed_unkIcon = createOverlayedIcon(
                baseComputationIcon, destroyed_unkOverlayIcon);
        // _----------------------------------------_//
        ImageIcon dt_unavailableIcon = createOverlayedIcon(
                baseDataTransferIcon, unavailableOverlayIcon);
        ImageIcon dt_createdIcon = createOverlayedIcon(baseDataTransferIcon,
                createdOverlayIcon);
        ImageIcon dt_runningIcon = createOverlayedIcon(baseDataTransferIcon,
                runningOverlayIcon);
        ImageIcon dt_doneIcon = createOverlayedIcon(baseDataTransferIcon,
                doneOverlayIcon);
        ImageIcon dt_failedIcon = createOverlayedIcon(baseDataTransferIcon,
                failedOverlayIcon);
        ImageIcon dt_destroyed_failedIcon = createOverlayedIcon(
                baseDataTransferIcon, destroyed_failedOverlayIcon);
        ImageIcon dt_destroyed_doneIcon = createOverlayedIcon(
                baseDataTransferIcon, destroyed_doneOverlayIcon);
        ImageIcon dt_destroyed_unkIcon = createOverlayedIcon(
                baseDataTransferIcon, destroyed_unkOverlayIcon);
        // _----------------------------------------_//
        TaskIconIdentifier c_unav = new TaskIconIdentifier(
                TaskType.COMPUTATION, TaskState.NOT_CONNECTED);
        TaskIconIdentifier c_created = new TaskIconIdentifier(
                TaskType.COMPUTATION, TaskState.CREATED);
        TaskIconIdentifier c_running = new TaskIconIdentifier(
                TaskType.COMPUTATION, TaskState.RUNNING);
        TaskIconIdentifier c_done = new TaskIconIdentifier(
                TaskType.COMPUTATION, TaskState.DONE);
        TaskIconIdentifier c_failed = new TaskIconIdentifier(
                TaskType.COMPUTATION, TaskState.FAILED);
        TaskIconIdentifier c_destroyed_done = new TaskIconIdentifier(
                TaskType.COMPUTATION, TaskState.DESTROYED_DONE);
        TaskIconIdentifier c_destroyed_failed = new TaskIconIdentifier(
                TaskType.COMPUTATION, TaskState.DESTROYED_FAILED);
        TaskIconIdentifier c_destroyed_unk = new TaskIconIdentifier(
                TaskType.COMPUTATION, TaskState.DESTROYED_UNKNOWN);
        TaskIconIdentifier dt_unav = new TaskIconIdentifier(
                TaskType.DATA_TRANSFER, TaskState.NOT_CONNECTED);
        TaskIconIdentifier dt_created = new TaskIconIdentifier(
                TaskType.DATA_TRANSFER, TaskState.CREATED);
        TaskIconIdentifier dt_running = new TaskIconIdentifier(
                TaskType.DATA_TRANSFER, TaskState.RUNNING);
        TaskIconIdentifier dt_done = new TaskIconIdentifier(
                TaskType.DATA_TRANSFER, TaskState.DONE);
        TaskIconIdentifier dt_failed = new TaskIconIdentifier(
                TaskType.DATA_TRANSFER, TaskState.FAILED);
        TaskIconIdentifier dt_destroyed_done = new TaskIconIdentifier(
                TaskType.DATA_TRANSFER, TaskState.DESTROYED_DONE);
        TaskIconIdentifier dt_destroyed_failed = new TaskIconIdentifier(
                TaskType.DATA_TRANSFER, TaskState.DESTROYED_FAILED);
        TaskIconIdentifier dt_destroyed_unk = new TaskIconIdentifier(
                TaskType.DATA_TRANSFER, TaskState.DESTROYED_UNKNOWN);
        // _----------------------------------------_//
        icons.put(c_unav, c_unavailableIcon);
        icons.put(c_created, c_createdIcon);
        icons.put(c_running, c_runningIcon);
        icons.put(c_done, c_doneIcon);
        icons.put(c_failed, c_failedIcon);
        icons.put(c_destroyed_done, c_destroyed_doneIcon);
        icons.put(c_destroyed_failed, c_destroyed_failedIcon);
        icons.put(c_destroyed_unk, c_destroyed_unkIcon);
        icons.put(dt_unav, dt_unavailableIcon);
        icons.put(dt_created, dt_createdIcon);
        icons.put(dt_running, dt_runningIcon);
        icons.put(dt_done, dt_doneIcon);
        icons.put(dt_failed, dt_failedIcon);
        icons.put(dt_destroyed_done, dt_destroyed_doneIcon);
        icons.put(dt_destroyed_failed, dt_destroyed_failedIcon);
        icons.put(dt_destroyed_unk, dt_destroyed_unkIcon);
        icons.put(new TaskIconIdentifier(TaskType.DUMMY, null), baseDummyIcon);
    }

    private static ImageIcon createOverlayedIcon(ImageIcon baseIcon,
            ImageIcon overlayIcon) {
        ImageIcon overlayedIcon = new ImageIcon();
        BufferedImage image = new BufferedImage(baseIcon.getIconWidth(),
                baseIcon.getIconHeight(), BufferedImage.TYPE_4BYTE_ABGR);
        Graphics2D imageGraphics = image.createGraphics();
        imageGraphics.drawImage(baseIcon.getImage(), 0, 0, null);
        imageGraphics.drawImage(overlayIcon.getImage(), image.getWidth()
                - overlayIcon.getIconWidth(), 0, null);
        imageGraphics.dispose();
        overlayedIcon.setImage(image);
        return overlayedIcon;
    }

}

class TaskIconIdentifier {
    private TaskType  type;
    private TaskState state;

    public TaskIconIdentifier(TaskType type, TaskState state) {
        this.type = type;
        this.state = state;
    }

    /**
     * @param type
     *            the type to set
     */
    public void setType(TaskType type) {
        this.type = type;
    }

    /**
     * @return the type
     */
    public TaskType getType() {
        return type;
    }

    /**
     * @param state
     *            the state to set
     */
    public void setState(TaskState state) {
        this.state = state;
    }

    /**
     * @return the state
     */
    public TaskState getState() {
        return state;
    }

    public boolean equals(Object o) {
        if (!(o instanceof TaskIconIdentifier))
            return false;
        TaskIconIdentifier id = (TaskIconIdentifier) o;
        if (this.type.equals(TaskType.DUMMY))
            return this.type.equals(id.getType());
        return this.type.equals(id.getType())
                && this.state.equals(id.getState());
    }

    public int hashCode() {
        int hashcode = type.hashCode();
        if (!type.equals(TaskType.DUMMY))
            hashcode += state.hashCode();
        return hashcode;
    }

    public String toString() {
        return "TaskIconIdentifier:[" + type + "," + state + "]";
    }
}
