/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.gui;

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.HashMap;
import java.util.Map;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JTextField;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 */
public class InputParamDialog extends JDialog {
    JButton                   ok, cancel;
    boolean                   ok_pressed       = false;
    Map<JLabel, JTextField>   params;
    private static final long serialVersionUID = -7061805186613640967L;

    public InputParamDialog(Frame f) {
        super(f, true);
    }

    public void init(String[] params) {
        Box pane = Box.createVerticalBox();
        this.params = new HashMap<JLabel, JTextField>();
        pane.add(Box.createRigidArea(new Dimension(1, 5)));
        for (int i = 0; i < params.length; i++) {
            Box innerPane = Box.createHorizontalBox();
            JLabel label = new JLabel(params[i]);
            JTextField field = new JTextField();
            field.setColumns(25);
            this.params.put(label, field);
            innerPane.add(label);
            innerPane.add(field);
            pane.add(innerPane);
            pane.add(Box.createRigidArea(new Dimension(1, 5)));
        }
        Box buttonPane = Box.createHorizontalBox();
        ok = new JButton("ok");
        ok.setMnemonic(KeyEvent.VK_ENTER);
        cancel = new JButton("cancel");
        cancel.setMnemonic(KeyEvent.VK_ESCAPE);
        ok.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                ok_pressed = true;
                InputParamDialog.this.setVisible(false);
            }

        });
        cancel.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                ok_pressed = false;
                InputParamDialog.this.setVisible(false);
            }

        });
        buttonPane.add(ok);
        buttonPane.add(cancel);
        pane.add(buttonPane);
        this.setContentPane(pane);
        this.pack();
    }

    public void init(String[] params, String[] defaults) {
        Box pane = Box.createVerticalBox();
        this.params = new HashMap<JLabel, JTextField>();
        pane.add(Box.createRigidArea(new Dimension(1, 5)));
        for (int i = 0; i < params.length; i++) {
            Box innerPane = Box.createHorizontalBox();
            JLabel label = new JLabel(params[i]);
            JTextField field = new JTextField();
            field.setText(defaults[i]);
            field.setColumns(25);
            this.params.put(label, field);
            innerPane.add(label);
            innerPane.add(field);
            pane.add(innerPane);
            pane.add(Box.createRigidArea(new Dimension(1, 5)));
        }
        Box buttonPane = Box.createHorizontalBox();
        ok = new JButton("ok");
        cancel = new JButton("cancel");
        ok.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                ok_pressed = true;
                InputParamDialog.this.setVisible(false);
            }

        });
        cancel.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                ok_pressed = false;
                InputParamDialog.this.setVisible(false);
            }

        });
        buttonPane.add(ok);
        buttonPane.add(cancel);
        pane.add(buttonPane);
        this.setContentPane(pane);
        this.pack();
    }

    public Map<String, String> getInput() {
        HashMap<String, String> input = new HashMap<String, String>();
        for (JLabel l : params.keySet()) {
            JTextField field = params.get(l);
            input.put(l.getText(), field.getText());
        }
        return input;
    }

    public boolean hasInput() {
        return ok_pressed;
    }
}
