/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.gui;

import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Collection;

import javax.swing.JComponent;

import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.control.AbstractGraphMousePlugin;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 */
public class PropertyGraphMousePlugin<V extends Task, E> extends
        AbstractGraphMousePlugin implements MouseListener {
    PropertyObserver<V> observer;
    Point               down, up, click;

    public PropertyGraphMousePlugin(int modifiers) {
        super(modifiers);
    }

    public PropertyGraphMousePlugin(int modifiers, PropertyObserver obs) {
        super(modifiers);
        this.observer = obs;
    }

    @Override
    public void mouseClicked(MouseEvent e) {
        VisualizationViewer<V, E> vv = (VisualizationViewer<V, E>) e
                .getSource();
        V vertex = vv.getPickSupport().getVertex(vv.getGraphLayout(),
                e.getPoint().x, e.getPoint().y);
        if (vertex != null) {
            observer.displayProperty(vertex);
            e.consume();
        }
    }

    @Override
    public void mouseEntered(MouseEvent e) {
        JComponent c = (JComponent) e.getSource();
        c.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));

    }

    @Override
    public void mouseExited(MouseEvent e) {
        JComponent c = (JComponent) e.getSource();
        c.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }

    @Override
    public void mousePressed(MouseEvent e) {
        down = e.getPoint();
    }

    @Override
    public void mouseReleased(MouseEvent e) {
        up = e.getPoint();
        VisualizationViewer<V, E> vv = (VisualizationViewer<V, E>) e
                .getSource();
        // observer.displayProperties(vv.getPickedVertexState().getPicked());
        if (getShapeFromPoints() != null) {
            Collection<V> vertices = vv.getPickSupport().getVertices(
                    vv.getGraphLayout(), getShapeFromPoints());
            observer.displayProperties(vertices);
        }
    }

    public Rectangle getShapeFromPoints() {
        if (down != null && up != null) {
            int width = (int) Math.abs(down.x - up.x);
            int height = (int) Math.abs(down.y - up.y);
            Point first = new Point(Math.min(down.x, up.x), Math.min(down.y,
                    up.y));
            return new Rectangle(first, new Dimension(width, height));
        }
        return null;
    }

}
