/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.gui;

import java.io.IOException;
import java.security.cert.X509Certificate;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import kgrid.raems.service.impl.Consts;
import kgrid.raems.stubs.RAEMSFactory.CreateResource;
import kgrid.raems.stubs.RAEMSFactory.CreateResourceResponse;
import kgrid.raems.stubs.RAEMSFactory.RAEMSFactoryPortType;
import kgrid.raems.stubs.RAEMSFactory.service.RAEMSFactoryServiceAddressingLocator;
import kgrid.raems.stubs.RAEMSService.RAEMSPortType;
import kgrid.raems.stubs.RAEMSService.service.RAEMSServiceAddressingLocator;

import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.globus.axis.util.Util;
import org.globus.delegation.DelegationUtil;
import org.globus.gsi.GSIConstants;
import org.globus.gsi.GlobusCredential;
import org.globus.wsrf.impl.security.authorization.Authorization;
import org.globus.wsrf.impl.security.authorization.HostAuthorization;
import org.globus.wsrf.impl.security.descriptor.ClientSecurityDescriptor;
import org.globus.wsrf.utils.AddressingUtils;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 */
public class RAEMSUtils implements Consts {

    public static EndpointReferenceType delegateCredential(String host)
            throws Exception {
        ClientSecurityDescriptor desc = getClientSecDesc();
        GlobusCredential credential = GlobusCredential.getDefaultCredential();
        String factoryUrl = ("https://" + host + ":" + CONTAINER_PORT + "/" + DELEGATION_FACTORY);
        int terminationTime = 60;
        int lifetime = terminationTime * 60; // lifetime in seconds
        EndpointReferenceType delegEpr = AddressingUtils
                .createEndpointReference(factoryUrl, null);
        X509Certificate[] certsToDelegateOn = DelegationUtil
                .getCertificateChainRP(delegEpr, desc);
        X509Certificate certToSign = certsToDelegateOn[0];
        return DelegationUtil.delegate(factoryUrl, credential, certToSign,
                lifetime, false, desc);
    }

    private static ClientSecurityDescriptor getClientSecDesc() {
        ClientSecurityDescriptor desc = new ClientSecurityDescriptor();
        Authorization authzVal = HostAuthorization.getInstance();
        Object authVal = GSIConstants.SIGNATURE;
        desc.setGSITransport((Integer) authVal);
        Util.registerTransport();
        desc.setAuthz(authzVal);
        return desc;
    }

    public static RAEMSPortType getRAEMSPortType(String ip) throws Exception {
        RAEMSFactoryServiceAddressingLocator factoryLocator = new RAEMSFactoryServiceAddressingLocator();
        RAEMSServiceAddressingLocator instanceLocator = new RAEMSServiceAddressingLocator();

        try {

            String factoryURI = "https://" + ip + ":" + CONTAINER_PORT + "/"
                    + RAEMS_FACTORY;
            EndpointReferenceType factoryEPR, instanceEPR;
            RAEMSFactoryPortType raemsFactory;
            RAEMSPortType raems;
            // 1: get Factory Port Type
            factoryEPR = new EndpointReferenceType();
            factoryEPR.setAddress(new Address(factoryURI));
            raemsFactory = factoryLocator
                    .getRAEMSFactoryPortTypePort(factoryEPR);
            // 2: Create resource and get endpoint reference of WS-Resource.
            // This resource is our "instance".
            CreateResource cr = new CreateResource();
            CreateResourceResponse createResponse = raemsFactory
                    .createResource(cr);
            instanceEPR = createResponse.getEndpointReference();

            // 3
            raems = instanceLocator.getRAEMSPortTypePort(instanceEPR);
            return raems;
        } catch (Exception e) {
        }
        return null;

    }// getRAEMSPortType

    public static Document loadFromFile(String path)
            throws ParserConfigurationException, SAXException, IOException {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder parser = factory.newDocumentBuilder();
        Document d = parser.parse(path);
        d.normalize();
        return d;
    }
}
