/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.gui;

import java.awt.Color;
import java.awt.Paint;

import org.apache.commons.collections15.Transformer;

import edu.uci.ics.jung.visualization.VisualizationViewer;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 */
public class StatefulTaskFillPaintTransformer<V extends Task> implements Transformer<V,Paint>{
	VisualizationViewer<V, ?> vv;
	
	public StatefulTaskFillPaintTransformer(VisualizationViewer<V, ?> vv) throws InterruptedException{
		this.vv=vv;
	}
	@Override
	public Paint transform(V v) {
		if(vv.getPickedVertexState().isPicked(v))
			return Color.orange;
		if(v.getStatus().equals("CREATED"))
			return Color.gray;
		else if(v.getStatus().equals("FAILED"))
			return Color.red;
		else if(v.getStatus().equals("DONE"))
			return Color.blue;
		else if(v.getStatus().equals("ACTIVE"))
			return Color.green;
		else if(v.getStatus().equals("DESTROYED"))
			return Color.black;
		else
			return Color.white;
	}
	
}
