/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.gui;

import java.util.LinkedList;
import java.util.Observable;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 */
public class Task extends Observable{
	private String name;
	private TaskState state;
	private LinkedList<TaskState> history;
	private TaskType type;
	
	public Task(String name){
		this.name=name;
		this.history=new LinkedList<TaskState>();
		this.setStatus(TaskState.NOT_CONNECTED);
	}
	
	public void setType(TaskType type){
		this.type=type;
	}
	
	public TaskType getType(){
		return this.type;
	}
	/**
	 * @param status the status to set
	 */
	public void setStatus(TaskState status) {
		if(status.equals(TaskState.DESTROYED_UNKNOWN) && history.contains(TaskState.DONE))
			this.state=TaskState.DESTROYED_DONE;
		else if(status.equals(TaskState.DESTROYED_UNKNOWN) && history.contains(TaskState.FAILED))
			this.state=TaskState.DESTROYED_FAILED;
		else
			this.state = status;
		history.add(state);
		this.setChanged();
		this.notifyObservers();
		this.clearChanged();
	}

	/**
	 * @return the status
	 */
	public TaskState getStatus() {
		return state;
	}

	/**
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}
	
	public String toString(){
		return this.name;
	}
	@Override
	public int hashCode() {
		return name.hashCode();
	}
	
	public boolean equals(Object o){
		if(!(o instanceof Task))
			return false;
		Task t=(Task)o;
		return t.getName().equals(this.name);
	}

	public String getHistory(){
		String hist="";
		for(TaskState s:history){
			hist+=s+"\n";
		}
		return hist;
	}
}
