/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl;

import java.util.List;

import kgrid.raems.service.impl.utils.KDSDownloadClient;
import kgrid.raems.service.impl.utils.KDSQueryClient;
import kgrid.raems.service.impl.utils.Session;

import org.apache.axis.message.addressing.EndpointReferenceType;
import org.globus.wsrf.NotifyCallback;
import org.globus.wsrf.core.notification.ResourcePropertyValueChangeNotificationElementType;
import org.oasis.wsrf.properties.ResourcePropertyValueChangeNotificationType;

/**
 * Simple listener used by KDSClient to receive notifications from the
 * KDSService
 * 
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class RAEMSListener implements NotifyCallback, Consts {

	private Object client;
	private Session session = null;

	public RAEMSListener(Object client) {
		this.client = client;
		if (client instanceof KDSQueryClient)
			session = ((KDSQueryClient) client).getSession();
		else if (client instanceof KDSDownloadClient)
			session = ((KDSDownloadClient) client).getSession();
		// else
		/*
		 * if(client instanceof KDSPublishClient) { KDSPublishClient
		 * pubClient=(KDSPublishClient) client;
		 * System.out.println("RAEMSListener da PublishClient="
		 * +pubClient.publishResource()
		 */
		// else
		// System.out.println("=====> NUOVA CLASSE INVOCANTE: "+client);
	}

	public void deliver(List topicPath, EndpointReferenceType producer,
			Object message) {
		// the client waits till a notification arrive
		synchronized (client) {
			// Notification of changes in KDS_Model delivered to the
			// RAEMSListener
			ResourcePropertyValueChangeNotificationElementType notificationElement;
			ResourcePropertyValueChangeNotificationType notification;
			String notificationResult = null;
			notificationElement = (ResourcePropertyValueChangeNotificationElementType) message;
			notification = notificationElement
					.getResourcePropertyValueChangeNotification();
			if (notification != null) {
				// XXX RAEMS: where and by whom is this nofitication generated?
				// XXX RAEMS: why is the localhost appended to the list?
				notificationResult = notification.getNewValue().get_any()[0]
						.getValue();
				if (session != null) {
					String notify = null;
					if (notificationResult.charAt(0) == '1') {
						notify = "#PREPROCESSING:Download completed$"
								+ notificationResult.substring(1);
					} else if (notificationResult.charAt(0) == '0') {
						notify = "#PREPROCESSING:Download failed$"
								+ notificationResult.substring(1);
					}
					// if(notificationResult.contains("SOURCE") &&
					// notificationResult.contains("DESTINATION"))
					// notify=
					if (notify != null) {
						session.setResourceModel(notify);
					} else {
						session.setResourceModel(notificationResult);
					}
				}
			}
			client.notify();
		}
	}
}
