/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl;

import org.globus.wsrf.Resource;
import org.globus.wsrf.ResourceIdentifier;
import org.globus.wsrf.ResourceProperties;
import org.globus.wsrf.ResourceProperty;
import org.globus.wsrf.ResourcePropertySet;
import org.globus.wsrf.Topic;
import org.globus.wsrf.TopicList;
import org.globus.wsrf.TopicListAccessor;
import org.globus.wsrf.impl.ResourcePropertyTopic;
import org.globus.wsrf.impl.SimpleResourceProperty;
import org.globus.wsrf.impl.SimpleResourcePropertySet;
import org.globus.wsrf.impl.SimpleTopicList;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class RAEMSResource implements Resource, ResourceIdentifier,
		ResourceProperties, TopicListAccessor {

	/* Resource Property set */
	private ResourcePropertySet propSet;

	/* Resource key. This uniquely identifies this resource. */
	private Object key;

	/* Resource properties */
	private ResourceProperty modelRP;


	/* Topic list */
	private TopicList topicList;

	/* Initializes RPs and returns a unique identifier for this resource */
	public Object initialize() throws Exception {
		this.key = new Integer(hashCode());
		this.propSet = new SimpleResourcePropertySet(
				Consts.RESOURCE_PROPERTIES);

		try {
			modelRP = new SimpleResourceProperty(Consts.RP_MODEL);
			modelRP.add("");

		} catch (Exception e) {
			throw new RuntimeException(e.getMessage());
		}

		/* Configure the Topics */
		this.topicList = new SimpleTopicList(this);

		modelRP = new ResourcePropertyTopic(modelRP);
		((ResourcePropertyTopic) modelRP).setSendOldValue(true);

		this.topicList.addTopic((Topic) modelRP);
		this.propSet.add(modelRP);

		return key;
	}

	/* Get/Setters for the RPs */
	
	public String getModel() {
		String x= (String) modelRP.get (0);
		return x;
	}

	public void setModel(String x) {
		modelRP.set(0,x);
	}

	/* Required by interface ResourceProperties */
	public ResourcePropertySet getResourcePropertySet() {
		return this.propSet;
	}

	/* Required by interface ResourceIdentifier */
	public Object getID() {
		return this.key;
	}

	/* Required by interface TopicListAccessor */
	public TopicList getTopicList() {
		return topicList;
	}
}
