/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl;

import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.xml.serialize.XMLSerializer;
import org.globus.exec.generated.JobDescriptionType;
import org.w3c.dom.Document;

/**
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class Utils implements Consts {

	public static String HOST_NAME;
	public static String HOST_IP;
	private static final String DATE_PATTERN = "yyyy.MM.dd HH:mm:ss";
	private static final SimpleDateFormat DATE_FORMATTER = new SimpleDateFormat(
			DATE_PATTERN);

	public static void print(String s) {
		String dateAndTime = DATE_FORMATTER.format(new Date());
		System.out.println(dateAndTime + " " + SERVICE_ACRONYM + " " + s);
	}

	static {
		try {
			HOST_NAME = (InetAddress.getLocalHost()).getCanonicalHostName();
			if (HOST_NAME == null || HOST_NAME.isEmpty()) {
				print("Error getting hostname from localhost (host name is empty)");
			}
			HOST_IP = (InetAddress.getLocalHost()).getHostAddress();
			if (HOST_IP == null || HOST_IP.isEmpty()) {
				print("Error getting host address from localhost (host address is empty)");
			}
		} catch (UnknownHostException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Returns true if the hosts, h1 and h2, are the same, false otherwise; the
	 * two hosts may be passed as IP address, hostname or fully qualified
	 * hostname indifferently.
	 * 
	 * @param h1
	 * @param h2
	 * @return true if the hosts, h1 and h2, are the same, false otherwise
	 * @throws UnknownHostException
	 */
	public static boolean sameHosts(String h1, String h2)
			throws UnknownHostException {
		if (h1.contains(":")) {
			h1 = h1.substring(0, h1.indexOf(":"));
		}
		if (h2.contains(":")) {
			h2 = h2.substring(0, h2.indexOf(":"));
		}
		String ip1 = InetAddress.getByName(h1).getHostAddress();
		String ip2 = InetAddress.getByName(h2).getHostAddress();
		return ip1.equals(ip2);
	}

	public static void printQuery(String[] query) {
		for (String string : query) {
			print("query[i] = " + string);
		}
	}

	public static void debugInfo() {
		System.out
				.println("*******************************************************");
		System.out.println();
		System.out.println("Host name and address = " + HOST_NAME + " "
				+ HOST_IP);
		System.out.println();
		System.out.println("User account name = "
				+ System.getProperty("user.name"));
		System.out.println("User home directory = "
				+ System.getProperty("user.home"));

		System.out
				.println("User working directory (i.e. the the directory where the globus container is started from) = "
						+ System.getProperty("user.dir"));

		System.out.println();
		System.out.println("Path of extension directory or directories = "
				+ System.getProperty("java.ext.dirs"));
		System.out.println("List of paths to search when loading libraries = "
				+ System.getProperty("java.library.path"));
		System.out.println("Java class path = "
				+ System.getProperty("java.class.path"));

		System.out.println();
		System.out.println("Java Virtual Machine implementation name = "
				+ System.getProperty("java.vm.name"));
		System.out.println("Java Runtime Environment version = "
				+ System.getProperty("java.version"));
		System.out.println("Java Runtime Environment vendor = "
				+ System.getProperty("java.vendor"));
		System.out.println();
		System.out
				.println("*******************************************************");
		System.out.println();
		System.out.println("Press any key to continue...");

		try {
			System.in.read();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public static String cleanString(String s) {
		return s.replaceAll("\n", "").replaceAll("\r", "").replaceAll("\t", "")
				.replaceAll(" ", "");
	}

	public static void printMessage(String title, String message) {
		System.out.println("\n .^^^^^^^^^^^^^^^^^^^ - " + title
				+ " - ^^^^^^^^^^^^^^^^^^^");
		System.out.println(" | " + message);
		System.out
				.println(" '_______________________________________________________________\n");
	}

	public static void printDocToScreen(Document doc) throws IOException {
		XMLSerializer serializer = new XMLSerializer();
		// Insert your PipedOutputStream here instead of System.out!
		serializer.setOutputByteStream(System.out);
		serializer.serialize(doc);
	}

	public static void printJobCommandLine(JobDescriptionType descriptionType) {
		print("Submitting GRAM job = ");
		System.out.print("\t" + descriptionType.getExecutable());
		for (String arg : descriptionType.getArgument()) {
			System.out.print(" " + arg);
		}
		System.out.println();
	}

	/**
	 * Returns true if the host parameter is equal to the localhost, false
	 * otherwise; the host may be passed as IP address, hostname or fully
	 * qualified hostname indifferently.
	 * 
	 * @param h1
	 * @param h2
	 * @return true if the host is the equal to the localhost, false otherwise
	 * @throws UnknownHostException
	 */
	public static boolean isLocalHost(String host) throws UnknownHostException {
		String ip = InetAddress.getByName(host).getHostAddress();
		return HOST_IP.equals(ip);
	}

}
