/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.execution;

import kgrid.raems.service.impl.utils.Session;
import kgrid.raems.service.impl.workflow.Task;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public abstract class ExecutionThread extends Thread {
    public enum State {
        CREATED, ACTIVE, DONE, FAILED, DESTROYED;
    }

    protected Task             task = null;
    protected State            state;
    protected ExecutionHandler executionHandler;
    protected Session          session;

    public ExecutionThread(Task t, ExecutionHandler executionHandler) {
        this.task = t;
        this.executionHandler = executionHandler;
    }

    public synchronized State getTaskState() {
        return state;
    }

    public Task getTask() {
        return task;
    }

    public void setSession(Session s) {
        this.session = s;
    }

    public abstract Throwable getErrorCause() throws Exception;

}
