/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.execution;

import java.net.URL;
import java.rmi.RemoteException;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.rpc.ServiceException;
import javax.xml.rpc.Stub;

import kgrid.raems.service.impl.Consts;
import kgrid.raems.service.impl.Utils;
import kgrid.raems.service.impl.utils.TransferStatusListener;

import org.apache.axis.message.addressing.EndpointReferenceType;
import org.globus.axis.util.Util;
import org.globus.gsi.GSIConstants;
import org.globus.rft.generated.BaseRequestType;
import org.globus.rft.generated.CreateReliableFileTransferInputType;
import org.globus.rft.generated.CreateReliableFileTransferOutputType;
import org.globus.rft.generated.DeleteRequestType;
import org.globus.rft.generated.OverallStatus;
import org.globus.rft.generated.RFTOptionsType;
import org.globus.rft.generated.ReliableFileTransferFactoryPortType;
import org.globus.rft.generated.ReliableFileTransferPortType;
import org.globus.rft.generated.Start;
import org.globus.rft.generated.TransferRequestType;
import org.globus.rft.generated.TransferType;
import org.globus.rft.generated.service.ReliableFileTransferFactoryServiceAddressingLocator;
import org.globus.rft.generated.service.ReliableFileTransferServiceAddressingLocator;
import org.globus.transfer.reliable.service.RFTConstants;
import org.globus.wsrf.NotificationConsumerManager;
import org.globus.wsrf.NotifyCallback;
import org.globus.wsrf.core.notification.ResourcePropertyValueChangeNotificationElementType;
import org.globus.wsrf.encoding.ObjectSerializer;
import org.globus.wsrf.encoding.SerializationException;
import org.globus.wsrf.impl.security.authorization.Authorization;
import org.globus.wsrf.impl.security.authorization.HostAuthorization;
import org.globus.wsrf.impl.security.descriptor.ClientSecurityDescriptor;
import org.oasis.wsn.Subscribe;
import org.oasis.wsn.SubscribeResponse;
import org.oasis.wsn.SubscriptionManager;
import org.oasis.wsn.TopicExpressionType;
import org.oasis.wsn.WSBaseNotificationServiceAddressingLocator;
import org.oasis.wsrf.lifetime.Destroy;
import org.oasis.wsrf.lifetime.ResourceNotDestroyedFaultType;
import org.oasis.wsrf.lifetime.ResourceUnknownFaultType;
import org.oasis.wsrf.lifetime.SetTerminationTime;
import org.oasis.wsrf.properties.ResourcePropertyValueChangeNotificationType;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class RAEMSFileTransferManager implements NotifyCallback, Consts {
	public static enum State {
		CREATED, ACTIVE, FAILED, PENDING, FINISHED;
	}

	private Throwable fault;
	private State state;
	private int terminationTime = 60;
	private NotificationConsumerManager consumer = null;
	private Object authVal = GSIConstants.SIGNATURE;
	private Authorization authzVal;
	private URL endpoint = null;
	private ReliableFileTransferFactoryPortType factoryPort;
	private ReliableFileTransferServiceAddressingLocator rftLocator;
	private ReliableFileTransferFactoryServiceAddressingLocator rftFactoryLocator;
	private ReliableFileTransferPortType rft;
	private EndpointReferenceType eprDelegationResource;
	protected String JobName;
	private boolean jobCompleted;
	private TransferStatusListener transferListener;
	private Object deliveryMonitor = new Object();
	private EndpointReferenceType notificationEndpoint = null;

	static {
		Util.registerTransport();
	}

	public RAEMSFileTransferManager(
			EndpointReferenceType eprDelegationResource, String JobName,
			TransferStatusListener listener) {
		this.fault = null;
		this.transferListener = listener;
		authzVal = HostAuthorization.getInstance();
		rftFactoryLocator = new ReliableFileTransferFactoryServiceAddressingLocator();
		this.eprDelegationResource = eprDelegationResource;
		this.JobName = JobName;
		this.jobCompleted = false;
		this.state = State.CREATED;
	}

	/*
	 * Performs a file transfer.
	 */
	public String performTransfer(String source, String dest) {
		try {
			TransferType[] transferArray = new TransferType[1];
			transferArray[0] = new TransferType();
			transferArray[0].setSourceUrl(source);
			transferArray[0].setDestinationUrl(dest);

			RFTOptionsType rftOptions = new RFTOptionsType();
			rftOptions.setNotpt(new Boolean(false)); // yes = thirdPartyTransfer
			rftOptions.setBinary(new Boolean(true));
			rftOptions.setBlockSize(new Integer(16000)); // Block size in bytes
			rftOptions.setTcpBufferSize(new Integer(16000)); // TCP Buffer size
			rftOptions.setIgnoreFilePermErr(true);
			// in bytes
			TransferRequestType request = new TransferRequestType();
			request.setRftOptions(rftOptions);
			request.setTransfer(transferArray);
			request.setTransferCredentialEndpoint(eprDelegationResource);
			// request.setAllOrNone(true);
			EndpointReferenceType rftepr = createRFT("https://" + Utils.HOST_IP
					+ ":" + CONTAINER_PORT + "/" + RFT_FACTORY, request);
			rftLocator = new ReliableFileTransferServiceAddressingLocator();
			ReliableFileTransferPortType rft = rftLocator
					.getReliableFileTransferPortTypePort(rftepr);
			setSecurity((Stub) rft);
			Subscribe request1 = new Subscribe();
			request1.setUseNotify(Boolean.TRUE);
			Map properties = new HashMap();
			properties.put("container.class",
					"org.globus.wsrf.container.GSIServiceContainer");
			consumer = NotificationConsumerManager.getInstance(properties);
			consumer.startListening();
			EndpointReferenceType consumerEPR = null;
			consumerEPR = consumer.createNotificationConsumer(this);
			request1.setConsumerReference(consumerEPR);
			TopicExpressionType topicExpression = new TopicExpressionType();
			topicExpression
					.setDialect("http://docs.oasis-open.org/wsn/2004/06/TopicExpression/Simple");
			topicExpression.setValue(RFTConstants.OVERALL_STATUS_RESOURCE);
			request1.setTopicExpression(topicExpression);
			SubscribeResponse subscribeResponse = rft.subscribe(request1);
			notificationEndpoint = subscribeResponse.getSubscriptionReference();
			Calendar termTime = Calendar.getInstance();
			termTime.add(Calendar.MINUTE, terminationTime);
			SetTerminationTime reqTermTime = new SetTerminationTime();
			reqTermTime.setRequestedTerminationTime(termTime);
			rft.setTerminationTime(reqTermTime);
			rft.start(new Start());
			waitForTransferCompletion(60000L);
			// return finish;
			String description = ObjectSerializer.toString(request,
					TransferRequestType.getTypeDesc().getXmlType());
			return description;
		} catch (Exception e) {
			this.setStatus(RAEMSFileTransferManager.State.FAILED);
			this.fault = e;
			e.printStackTrace();
		}
		return "ERROR in RAEMSFileTransferManager/performTransfer: see stack trace";
	}

	private synchronized void waitForTransferCompletion(long term_time)
			throws InterruptedException {
		while (!jobCompleted) {
			this.wait(term_time);
		}

	}

	public void destroy() throws ResourceNotDestroyedFaultType,
			ResourceUnknownFaultType, RemoteException, SerializationException,
			ServiceException {
		if (notificationEndpoint != null) {
			SubscriptionManager subscriptionPort = new WSBaseNotificationServiceAddressingLocator()
					.getSubscriptionManagerPort(this.notificationEndpoint);
			setSecurity((Stub) subscriptionPort);
			// unbinding notification
			subscriptionPort.destroy(new Destroy());
		}
		if (rft != null) {
			rft.destroy(new Destroy());
		}
	}

	/*
	 * Creates an RFT resource.
	 */
	private EndpointReferenceType createRFT(String rftFactoryAddress,
			BaseRequestType request) throws Exception {
		endpoint = new URL(rftFactoryAddress);
		factoryPort = rftFactoryLocator
				.getReliableFileTransferFactoryPortTypePort(endpoint);
		CreateReliableFileTransferInputType input = new CreateReliableFileTransferInputType();
		if (request instanceof TransferRequestType) {
			input.setTransferRequest((TransferRequestType) request);
		} else {
			input.setDeleteRequest((DeleteRequestType) request);
		}
		Calendar termTime = Calendar.getInstance();
		termTime.add(Calendar.HOUR, 1);
		input.setInitialTerminationTime(termTime);
		setSecurity((Stub) factoryPort);
		CreateReliableFileTransferOutputType response = factoryPort
				.createReliableFileTransfer(input);
		return response.getReliableTransferEPR();
	}

	private void setSecurity(Stub stub) {
		stub._setProperty("clientDescriptor", getClientSecDesc());
	}

	private ClientSecurityDescriptor getClientSecDesc() {
		ClientSecurityDescriptor desc = new ClientSecurityDescriptor();
		desc.setGSITransport((Integer) authVal);
		Util.registerTransport();
		desc.setAuthz(authzVal);
		return desc;
	}

	public State getStatus() {
		synchronized (deliveryMonitor) {
			return this.state;
		}
	}

	public void setStatus(State newState) {
		synchronized (deliveryMonitor) {
			if (!this.state.equals(newState)) {
				this.state = newState;
				if (state.equals(State.FAILED) || state.equals(State.FINISHED))
					this.jobCompleted = true;
				transferListener.statusChanged(this);
			}
		}
	}

	public void deliver(List topicPath, EndpointReferenceType producer,
			Object message) {
		ResourcePropertyValueChangeNotificationType changeMessage = ((ResourcePropertyValueChangeNotificationElementType) message)
				.getResourcePropertyValueChangeNotification();
		// DataTransfer Notification Arrived
		synchronized (deliveryMonitor) {
			try {
				if (changeMessage != null) {
					OverallStatus overallStatus = ((OverallStatus) (changeMessage
							.getNewValue().get_any()[0]
							.getValueAsType(
									RFTConstants.OVERALL_STATUS_RESOURCE,
									Class.forName("org.globus.rft.generated.OverallStatus"))));
					int finished = overallStatus.getTransfersFinished();
					int failed = overallStatus.getTransfersFailed();
					int active = overallStatus.getTransfersActive();
					int pending = overallStatus.getTransfersPending();

					// if (finished >= transferCount)
					// this.setStatus(RAEMSFileTransferManager.State.FINISHED);
					// if (active >= 1)
					// this.setStatus(RAEMSFileTransferManager.State.ACTIVE);
					// if (pending >= 1
					// && (pending + finished >= transferCount || pending
					// + failed >= transferCount))
					// this.setStatus(RAEMSFileTransferManager.State.PENDING);
					// if (failed >= 1 && finished + failed >= transferCount)
					// this.setStatus(RAEMSFileTransferManager.State.FAILED);
					if (finished == 1) {
						this.setStatus(RAEMSFileTransferManager.State.FINISHED);
					}
					if (active == 1) {
						this.setStatus(RAEMSFileTransferManager.State.ACTIVE);
					}
					if (pending == 1) {
						this.setStatus(RAEMSFileTransferManager.State.PENDING);
					}
					if (failed == 1) {
						// FIXME catturare il messaggio d'errore e concatenarlo
						// alla stringa del messaggio d'errore FAILED...
						this.fault = new Throwable(overallStatus.getFault()
								.toString());
						this.setStatus(RAEMSFileTransferManager.State.FAILED);
					}

				}
			} catch (Exception e) {
				this.fault = e;
				this.setStatus(RAEMSFileTransferManager.State.FAILED);
			}
		}
		synchronized (this) {
			notify();
		}
	}

	public Throwable getFault() {
		return fault;
	}

}
