/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.scheduler;

import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.Map;

import javax.xml.rpc.ServiceException;

import kgrid.raems.service.impl.Consts;
import kgrid.raems.service.impl.exception.ConnectionException;
import kgrid.raems.service.impl.utils.KDSClient;
import kgrid.raems.service.impl.utils.KDSResultSet;
import kgrid.raems.service.impl.utils.KDSUrl;
import kgrid.raems.service.impl.utils.ResourceLookupInfo;
import kgrid.raems.service.impl.workflow.Job;
import kgrid.raems.service.impl.workflow.Resource;

import org.globus.wsrf.NoSuchResourceException;
import org.globus.wsrf.encoding.SerializationException;
import org.globus.wsrf.query.QueryException;
import org.oasis.wsrf.lifetime.ResourceNotDestroyedFaultType;
import org.oasis.wsrf.lifetime.ResourceUnknownFaultType;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class ResourceLookup implements Consts {
	public static Map<Resource, ResourceLookupInfo> lookup(KDSClient kdsClient,
			Job abstractEP) throws ResourceNotDestroyedFaultType,
			ResourceUnknownFaultType, NoSuchResourceException, QueryException,
			SerializationException, RemoteException, ConnectionException,
			InterruptedException, ServiceException {
		HashMap<Resource, ResourceLookupInfo> ret = new HashMap<Resource, ResourceLookupInfo>();
		for (Resource r : abstractEP.getResourceCache().getAll()) {
			KDSResultSet resultSet;
			ResourceLookupInfo resourceLookupInfo;
			if (r.isAbstract() && !r.getQuery().isEmpty()) {
				if (!r.isPublished()) {
					resultSet = kdsClient.makeQuery(r.getQuery());
				} else {
					resultSet = new KDSResultSet();
					resultSet.addResult(new KDSUrl(r.getHref()));
				}
				if (resultSet != null) {
					resourceLookupInfo = new ResourceLookupInfo();
					resourceLookupInfo.setResource(r);
					resourceLookupInfo.setResultSet(resultSet);
					ret.put(r, resourceLookupInfo);
				}
			}
		}
		return ret;
	}
}
