/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.scheduler;

import java.util.HashMap;
import java.util.Map;

import kgrid.raems.service.impl.Utils;
import kgrid.raems.service.impl.utils.KDSClient;
import kgrid.raems.service.impl.utils.KDSUrl;
import kgrid.raems.service.impl.utils.ResourceLookupInfo;
import kgrid.raems.service.impl.workflow.Job;
import kgrid.raems.service.impl.workflow.Resource;

import org.globus.wsrf.NoSuchResourceException;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class SimpleMapper implements Mapper {
	ResourceMapper resourceMapper;

	public SimpleMapper(ResourceMapper resourceMapper) {

		this.resourceMapper = resourceMapper;
	}

	public Map<Resource, KDSUrl> map(KDSClient client, Job abstractEP,
			Map<Resource, ResourceLookupInfo> resourceInfos)
			throws NoSuchResourceException {

		HashMap<Resource, KDSUrl> resourcesMapping = new HashMap<Resource, KDSUrl>();
		for (Resource r : abstractEP.getResourceCache().getAll()) {

			KDSUrl url = null;
			if (r.isAbstract()) {
				if (r.isPublished()) {
					url = new KDSUrl(r.getHref());
					resourcesMapping.put(r, url);

				} else {
					ResourceLookupInfo info = resourceInfos.get(r);
					if (info != null) {
						url = resourceMapper.map(info.getResultSet());
					}
				}
			}
			if (url != null) {
				resourcesMapping.put(r, url);
			}
		}
		for (Map.Entry<Resource, KDSUrl> entry : resourcesMapping.entrySet()) {
			Resource key = entry.getKey();
			KDSUrl value = entry.getValue();
			Utils.print(key.getId() + " " + key.getName()
					+ " is assigned to ===> " + value.getURL());
		}
		return resourcesMapping;
	}
}
