/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.utils;

import java.rmi.RemoteException;

import javax.xml.rpc.ServiceException;

import kgrid.kds.stubs.KDSService.DownloadParameters;
import kgrid.raems.service.impl.Consts;
import kgrid.raems.service.impl.Utils;
import kgrid.raems.service.impl.exception.ConnectionException;
import kgrid.raems.service.impl.exception.DownloadException;
import kgrid.raems.service.impl.exception.GeneralException;

import org.globus.wsrf.encoding.SerializationException;
import org.globus.wsrf.impl.notification.SubscriptionCreationException;
import org.oasis.wsrf.lifetime.ResourceNotDestroyedFaultType;
import org.oasis.wsrf.lifetime.ResourceUnknownFaultType;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class KDSClient implements Consts {

    private KDSDownloadClient downloadClient = null;
    private KDSQueryClient    queryClient    = null;
    private KDSPublishClient  publishClient  = null;
    private Session           session        = null;

    public KDSClient(Session s) {
        downloadClient = new KDSDownloadClient(s);
        queryClient = new KDSQueryClient(s);
        publishClient = new KDSPublishClient();
        this.session = s;
    }

    public Session getSession() {
        return session;
    }

    public KDSResultSet makeQuery(Query q)
            throws ResourceNotDestroyedFaultType, ResourceUnknownFaultType,
            SerializationException, RemoteException, ConnectionException,
            InterruptedException, ServiceException {
        return queryClient.makeQuery(q);
    }

    /**
     * WHAT HAPPENS IF 2 FILE WITH THE SAME NAME ARE REQUESTED TO BE
     * DOWNLOADED??
     * 
     * @param url
     * @return
     * @throws SubscriptionCreationException
     * @throws ConnectionException
     * @throws GeneralException
     * @throws DownloadException
     */
    public String download(KDSUrl url) throws SubscriptionCreationException,
            ConnectionException, GeneralException, DownloadException {
        String result;
        try {
            DownloadParameters localParameters = new DownloadParameters();
            localParameters.setKdsURL(url.getURL());
            // session.setResourceModel(url.getURL()); // SETTAGGIO DELLA
            // RISORSA SCELTA PER IL DOWNLOAD
            localParameters.setEprCredential(session.getClientCredentials());
            String uploadPath = GSIFTP_PROTOCOL + Utils.HOST_IP + ":"
                    + GSI_PORT + SESSION_PATH + session.getSessionID();

            localParameters.setPathUpload(uploadPath);

            result = downloadClient.download(localParameters);
        } catch (SubscriptionCreationException e) {
            throw e;
        } catch (ConnectionException e) {
            throw e;
        } catch (GeneralException e) {
            throw e;
        } catch (Exception e) {
            throw new GeneralException("", e);
        }
        int resultInt = Integer.parseInt(result);

        if (resultInt <= 0) {
            throw new DownloadException("kds download failed.");
        }

        return SESSION_PATH + session.getSessionID();

    }

    public String publish(String resourceURL)
            throws SubscriptionCreationException, ConnectionException,
            GeneralException {
        return publishClient.publishResource(resourceURL);
    }

    public String publish(String destinationHost, String resourceURL)
            throws SubscriptionCreationException, ConnectionException,
            GeneralException {

        return publishClient.publishResource(destinationHost, resourceURL);
    }
}
