/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.utils;

import java.rmi.RemoteException;

import kgrid.kds.stubs.KDSService.DownloadParameters;
import kgrid.kds.stubs.KDSService.KDSPortType;
import kgrid.raems.service.impl.Consts;
import kgrid.raems.service.impl.RAEMSListener;
import kgrid.raems.service.impl.exception.ConnectionException;
import kgrid.raems.service.impl.exception.GeneralException;

import org.globus.wsrf.impl.notification.SubscriptionCreationException;
import org.oasis.wsn.SubscribeResponse;
import org.oasis.wsrf.lifetime.Destroy;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class KDSDownloadClient implements Consts {

    private Session session = null;

    public KDSDownloadClient(Session session) {
        this.session = session;
    }

    public Session getSession() {
        return session;
    }

    public String download(DownloadParameters localParameters)
            throws ConnectionException, SubscriptionCreationException,
            GeneralException {
        try {
            RAEMSListener raemsListener = new RAEMSListener(this);
            KDSPortType kds;
            kds = KDSConnectionUtils.getKDSPortType();
            SubscribeResponse response = KDSConnectionUtils
                    .subscribeToKDSModelTopic(raemsListener, kds);

            DownloadThread downloadThread = new DownloadThread(kds,
                    localParameters);
            downloadThread.start();

            String notify = "#PREPROCESSING:Download started$ "
                    + localParameters.getKdsURL();
            session.setResourceModel(notify);

            synchronized (this) {
                this.wait();
            }
            KDSConnectionUtils.destroySubscription(response);
            kds.destroy(new Destroy());
            if (downloadThread.getFault() != null) {
                throw downloadThread.getFault();
            }
            return downloadThread.getResult();

        } catch (ConnectionException e) {
            throw e;
        } catch (SubscriptionCreationException e) {
            throw e;
        } catch (Throwable e) {
            throw new GeneralException("", e);
        }
    }

    class DownloadThread extends Thread implements KDSClientThread {

        private KDSPortType        kds;
        private DownloadParameters kdsParameters;
        private String             result = null;
        private Throwable          fault  = null;

        public DownloadThread(KDSPortType kdsport,
                DownloadParameters kdsParameters) {
            this.kds = kdsport;
            this.kdsParameters = kdsParameters;
        }

        public void run() throws RuntimeException {
            try {
                result = kds.downloadResource(kdsParameters);
            } catch (RemoteException e) {
                fault = e;
            }
        }

        public String getResult() {
            return result;
        }

        @Override
        public Throwable getFault() {
            return fault;
        }
    }

}
