/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.utils;

import java.util.LinkedList;
import java.util.List;

import kgrid.kds.stubs.KDSService.ArrayOfString;
import kgrid.raems.service.impl.Consts;
import kgrid.raems.service.impl.Utils;
import kgrid.raems.service.impl.exception.EmptyResultSetException;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class KDSResultSet implements Consts {
	protected LinkedList<KDSUrl> urls;

	public KDSResultSet(ArrayOfString result) throws EmptyResultSetException {
		if (result == null) {
			Utils.print("Result == null");
			throw new EmptyResultSetException();
		}
		urls = new LinkedList<KDSUrl>();
		int count = 0;
		for (String s : result.getX()) {
			if (s != null) {
				urls.add(new KDSUrl(s));
				count++;
			}
		}
		if (count <= 0) {
			throw new EmptyResultSetException();
		}
	}

	public KDSResultSet() {
		urls = new LinkedList<KDSUrl>();
	}

	public KDSUrl getFirst() throws EmptyResultSetException {
		if (urls.isEmpty()) {
			throw new EmptyResultSetException();
		}
		return urls.get(0);
	}

	public KDSUrl getLast() throws EmptyResultSetException {
		if (urls.isEmpty()) {
			throw new EmptyResultSetException();
		}
		return urls.getLast();

	}

	public KDSUrl get(int i) throws EmptyResultSetException {
		if (urls.size() <= i) {
			throw new EmptyResultSetException();
		}
		return urls.get(i);

	}

	public List<KDSUrl> getURLs() {
		return urls;
	}

	public int size() {
		return urls.size();
	}

	public void addResult(KDSUrl url) {
		urls.add(url);
	}

	@Override
	public boolean equals(Object o) {
		if (!(o instanceof KDSResultSet)) {
			return false;
		}
		KDSResultSet set = (KDSResultSet) o;
		for (KDSUrl url : set.urls) {
			if (!urls.contains(url)) {
				return false;
			}
		}
		for (KDSUrl url : urls) {
			if (!set.urls.contains(url)) {
				return false;
			}
		}
		return true;
	}

	@Override
	public int hashCode() {
		int hash = 7;
		for (KDSUrl url : urls) {
			hash = 31 * hash + url.hashCode();
		}
		return hash;
	}
}
