/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.utils;

import java.security.cert.X509Certificate;

import kgrid.raems.service.impl.Consts;
import kgrid.raems.service.impl.exception.CreateEndpointReferenceException;

import org.apache.axis.message.addressing.EndpointReferenceType;
import org.globus.axis.util.Util;
import org.globus.delegation.DelegationException;
import org.globus.delegation.DelegationUtil;
import org.globus.delegation.service.DelegationResource;
import org.globus.gsi.GSIConstants;
import org.globus.gsi.GlobusCredential;
import org.globus.wsrf.impl.security.authorization.Authorization;
import org.globus.wsrf.impl.security.authorization.HostAuthorization;
import org.globus.wsrf.impl.security.descriptor.ClientSecurityDescriptor;
import org.globus.wsrf.utils.AddressingUtils;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class Security {
	private static int terminationTime = 60;

	private static Object authVal = GSIConstants.SIGNATURE;

	private static ClientSecurityDescriptor getClientSecDesc() {
		ClientSecurityDescriptor desc = new ClientSecurityDescriptor();
		Authorization authzVal = HostAuthorization.getInstance();
		desc.setGSITransport((Integer) authVal);
		Util.registerTransport();
		desc.setAuthz(authzVal);
		return desc;
	}// getClientSecDesc

	public static EndpointReferenceType delegateCredential(String host,
			int port, EndpointReferenceType eprDelegationResource)
			throws DelegationException, CreateEndpointReferenceException {
		ClientSecurityDescriptor desc = getClientSecDesc();
		DelegationResource dr = DelegationUtil
				.getDelegationResource(eprDelegationResource);
		GlobusCredential credential = dr.getCredential();
		String factoryUrl = ("https://" + host + ":" + port + "/" + Consts.DELEGATION_FACTORY);
		int lifetime = terminationTime * 60; // lifetime in seconds
		EndpointReferenceType delegEpr = null;
		try {
			delegEpr = AddressingUtils
					.createEndpointReference(factoryUrl, null);
		} catch (Exception e) {
			throw new CreateEndpointReferenceException(e.getMessage());
		}
		X509Certificate[] certsToDelegateOn = DelegationUtil
				.getCertificateChainRP(delegEpr, desc);
		X509Certificate certToSign = certsToDelegateOn[0];
		EndpointReferenceType x = null;

		x = DelegationUtil.delegate(factoryUrl, credential, certToSign,
				lifetime, false, desc);

		return x;
	}// delegateCredential
}
