/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.utils;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintStream;
import java.io.PrintWriter;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import kgrid.raems.service.impl.Consts;

import org.apache.xml.serialize.DOMSerializer;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 * Utility class for XML Documents
 * 
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class XmlUtils implements Consts {
    /**
     * Print a org.w3c.dom.Document on the System.out stream
     * 
     * @param doc
     * @throws IOException
     */
    public static void printDocument(Document doc) throws IOException {

        PrintWriter pw = new PrintWriter(System.out);
        DOMSerializer ser = new XMLSerializer(pw, new OutputFormat(doc,
                "UTF-8", true));
        ser.serialize(doc);

    }

    public static void printDocument(Document doc, PrintStream stream)
            throws IOException {
        DOMSerializer ser = new XMLSerializer(stream, new OutputFormat(doc,
                "UTF-8", true));
        ser.serialize(doc);

    }

    /**
     * Save a org.w3c.dom.Document object on a specified file
     * 
     * @param doc
     * @param fileName
     * @throws IOException
     */
    public static void saveDocument(Document doc, String fileName)
            throws IOException {
        String finalFileName = "";
        if (fileName.endsWith(".xml"))
            finalFileName = fileName;
        else
            finalFileName = fileName + ".xml";
        PrintWriter pw = new PrintWriter(new FileWriter(finalFileName));
        DOMSerializer ser = new XMLSerializer(pw, new OutputFormat(doc,
                "UTF-8", true));
        ser.serialize(doc);
    }

    public static void saveStringAsXMLDocument(String doc, String fileName)
            throws SAXException, IOException, ParserConfigurationException {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder parser = factory.newDocumentBuilder();
        ByteArrayInputStream inStream = new ByteArrayInputStream(doc.getBytes());
        Document d = parser.parse(inStream);
        saveDocument(d, fileName);
        inStream.close();
    }

    public static Document loadFromFile(String path)
            throws ParserConfigurationException, SAXException, IOException {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder parser = factory.newDocumentBuilder();
        Document d = parser.parse(path);
        d.normalize();
        return d;
    }

    public static void main(String[] arg) {
        String doc = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
                + "\n"
                + "<DataMiningSoftware name=\"Splitter\">"
                + "\n"
                + "<Description>"
                + "\n"
                + " <KindOfData>arff data</KindOfData>"
                + "\n"
                + "  <KindOfKnowledge>preProcessing</KindOfKnowledge>"
                + "\n"
                + "  <KindOfTechnique>unsepervised</KindOfTechnique>"
                + "\n"
                + "  <DrivingMethod></DrivingMethod>"
                + "\n"
                + "</Description>"
                + "\n"
                + "<Usage>"
                + "\n"
                + "<KindOfSoftware>executable</KindOfSoftware>"
                + "\n"
                + "<Invocation>/usr/bin/java</Invocation>"
                + "\n"
                + "<Location>" + KMR_PATH + "tools/splitter.jar</Location>"
                + "\n"
                + "<ManualPath>/lib/datamining.doc</ManualPath>"
                + "\n"
                + "<OutputFolder>/tmp/</OutputFolder>"
                + "\n"
                + "<Args>"
                + "\n"
                + "<Arg name=\"jar\" type=\"string\" default=\"" + KMR_PATH + "tools/splitter.jar\" mandatory=\"true\">-jar</Arg>"
                + "\n"
                + "<Arg name=\"InputDataSet\" type=\"string\">-i</Arg>"
                + "\n"
                + "<Arg name=\"SplitDataSet\" type=\"string\" multivalue=\"true\" >-o</Arg>"
                + "\n"
                + "   </Args>"
                + "\n"
                + "    <HostName>andrea-laptop</HostName>"
                + "\n"
                + "   <DocumentationURL>http://jupiter.deis.unical.it/documentation/clustering.htm</DocumentationURL>"
                + "\n" + "  </Usage>" + "\n" + "</DataMiningSoftware>" + "\n";
        try {
            saveStringAsXMLDocument(doc, System.getProperty("user.home")
                    + File.separator + "prova.xml");
        } catch (SAXException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (ParserConfigurationException e) {
            e.printStackTrace();
        }
    }

}
