/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.workflow;

import kgrid.raems.service.impl.utils.ID;
import kgrid.raems.service.impl.utils.Query;

import org.globus.wsrf.query.QueryException;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public abstract class AbstractResource implements Resource {
    protected ID id;

    protected String name, href, title;

    public AbstractResource() {
        id = null;
        name = "";
        href = "";
        title = "";
    }

    public String getHref() {
        return href;
    }

    public ID getId() {
        return id;
    }

    public String getName() {
        return name;
    }

    public Query getQuery() throws QueryException {
        return new Query();
    }

    public String getTitle() {
        return title;
    }

    public boolean isAbstract() {
        if (getHostName() == null || getHostName().isEmpty()
                || getPhysicalLocation() == null
                || getPhysicalLocation().isEmpty()) {
            return true;
        }
        return false;
    }

    public boolean isPublished() {
        if (href != null && !href.isEmpty() && title != null
                && !title.isEmpty()) {
            return true;
        }
        return false;
    }

    public boolean isReferenced() {
        return id.isIDRef();
    }

    public void setId(ID id) {
        if (id == null) {
            throw new RuntimeException("resource must have id");
        }
        this.id = id;
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setPublicInformation(String href, String title) {
        this.href = href;
        this.title = title;
    }

    public boolean equals(Object obj) {
        if (obj instanceof Resource) {
            return id.equals(((Resource) obj).getId());
        }
        return false;
    }

    public int hashCode() {
        return id.hashCode();
    }

    public abstract String toString();

}
