/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.workflow;

import java.util.ArrayList;
import java.util.List;

import kgrid.raems.service.impl.workflow.plugin.Arg;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class ArgumentList extends ArrayList<Arg> {

    private static final long serialVersionUID = -1775716439467369332L;

    public ArgumentList getMandatoryArguments() {
        ArgumentList ret = new ArgumentList();
        for (Arg arg : this)
            if (arg.isMandatory())
                ret.add(arg);
        return ret;
    }

    public List<String> getArgumentValue(String argName) {
        for (Arg arg : this) {
            if (arg.getName().equals(argName))
                return arg.getValues();
        }
        return null;
    }

    public ArgumentList getArgumentByName(String argName) {
        ArgumentList args = new ArgumentList();
        for (Arg arg : this) {
            if (arg.getName().equals(argName))
                args.add(arg);
        }
        return args;
    }

    public Arg getArgumentByValue(String value) {
        for (Arg arg : this) {
            if (arg.getValues() != null && arg.getValues().size() > 0)
                if (arg.getValues().contains(value))
                    return arg;
        }
        return null;
    }

}
