/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.workflow;

import java.util.HashSet;
import java.util.Set;

import kgrid.raems.service.impl.utils.SetUtils;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class Attribute implements Cloneable {

    private String        name, type;
    private Set<Property> properties;

    public Attribute() {

        name = "";
        type = "";
        properties = new HashSet<Property>();
    }

    public void addProperty(Property property) {
        properties.add(property);
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getType() {
        return type;
    }

    public void setType(String type) {
        this.type = type;
    }

    public Set<Property> getProperties() {
        return properties;
    }

    public void setProperties(Set<Property> properties) {
        this.properties = properties;
    }

    public String toString() {
        String s = "";
        if (name != null && !name.equals(""))
            s += "name: " + name + "\n";
        if (type != null && !type.equals(""))
            s += "type: " + type + "\n";
        if (properties != null && properties.size() > 0) {
            s += "properties:\n";
            for (Property p : properties)
                s += p.toString() + "\n";
        }
        return s;
    }

    public Object clone() throws CloneNotSupportedException {

        Attribute attribute = new Attribute();
        attribute.setName(new String(this.name));
        attribute.setType(new String(this.type));

        try {
            Set<Property> set = (Set<Property>) SetUtils.cloneSet(properties);
            attribute.setProperties(set);
        } catch (Throwable t) {
            throw new CloneNotSupportedException();
        }

        return attribute;
    }

}
