/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.workflow;

import java.util.Collection;
import java.util.LinkedList;

import javax.xml.parsers.ParserConfigurationException;

import kgrid.raems.service.impl.workflow.constants.Constants;
import kgrid.raems.service.impl.workflow.constants.JobConstants;

import org.w3c.dom.Element;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class DataTransfer extends Task {

	private static final long serialVersionUID = 1L;
	private Resource input;
	private Resource output;
	private String src, dest;
	private boolean publish;

	public DataTransfer(String ID) {
		super(ID);

		src = null;
		dest = null;
		publish = false;
	}

	public void addInput(Resource input) {
		if (input.getId() == null)
			throw new RuntimeException(input.getClass() + " must have an id");
		if (this.input != null)
			throw new RuntimeException(
					"A DataTransfer can't have more then 1 input");
		this.input = input;
	}

	public String getSource() {
		return src;
	}

	public void setSource(String src) {
		this.src = src;
	}

	public String getDestination() {
		return dest;
	}

	public void setDestination(String dest) {
		this.dest = dest;
	}

	public void setPublishResource(boolean publish) {
		this.publish = publish;
	}

	public boolean isPublishResource() {
		return publish;
	}

	public boolean isAbstract() {
		if (src.equals(""))
			return true;
		return false;
	}

	public String toString() {
		return "{Datatransfer:\n source host:" + src + "\ndestination host:"
				+ dest + "\nname :" + getName() + "\n" + "Inputs: " + input
				+ "\nOutputs:" + output + "}\n";
	}

	public void replaceResource(Resource from, Resource to) {
		if (input.getId().equals(from.getId())) {

			this.input = to;
		} else if (output.getId().equals(from.getId())) {
			this.output = to;
		}

	}

	public Element serialize() throws ParserConfigurationException {
		Serializer rootSer = super.serializer();
		Serializer serializer = new Serializer("DataTransfer");
		serializer.addAsAttribute(Constants.Name, this.getName());
		serializer.addAsAttribute(Constants.Source, this.src);
		serializer.addAsAttribute(Constants.Destination, dest);

		Serializer source_ser = new Serializer(JobConstants.Input);
		source_ser.addAsAttribute(Constants.Href, input.getHref());
		source_ser.addAsAttribute(Constants.Title, input.getTitle());
		source_ser.addAsChildElement(input.serialize());

		serializer.addAsChildElement(source_ser.serializedElement());

		rootSer.addAsChildElement(serializer.serializedElement());
		return rootSer.serializedElement();
	}

	public void addOutput(Resource output) {
		if (output.getId() == null)
			throw new RuntimeException(output.getClass() + " must have an id");
		if (this.output != null)
			throw new RuntimeException(
					"A DataTransfer can't have more then 1 output");
		this.output = output;

	}

	public Collection<Resource> getInputs() {
		LinkedList<Resource> ret = new LinkedList<Resource>();
		if (input != null)
			ret.add(input);
		return ret;
	}

	public Collection<Resource> getOutputs() {
		LinkedList<Resource> ret = new LinkedList<Resource>();
		if (output != null) {
			ret.add(output);
		}
		return ret;

	}

	public Collection<Resource> getResources() {
		LinkedList<Resource> ret = new LinkedList<Resource>();
		if (input != null)
			ret.add(input);
		if (output != null)
			ret.add(output);
		return ret;
	}
}
