/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.workflow;

import kgrid.raems.service.impl.workflow.constants.ParserConstants;
import kgrid.raems.service.impl.workflow.parser.Builder;
import kgrid.raems.service.impl.workflow.parser.Director;
import kgrid.raems.service.impl.workflow.parser.Parser;
import kgrid.raems.service.impl.workflow.parser.ParserTable;

import org.w3c.dom.Node;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class Deserializer {
    PluginDescription desc;

    public Deserializer(PluginDescription pluginDescription) {
        this.desc = pluginDescription;
    }

    public Object deserialize(Node n) throws Exception {
        Class<? extends ParserConstants> pconstclass = (Class<? extends ParserConstants>) Class
                .forName(desc.getParserconstants());
        ParserTable parserTable = new ParserTable(pconstclass);
        Class<? extends Builder> builderClass = (Class<? extends Builder>) Class
                .forName(desc.getBuildername());
        Builder b = builderClass.newInstance();
        Class<? extends Parser> parserClass = (Class<? extends Parser>) Class
                .forName(desc.getParsername());
        Parser p = parserClass.newInstance();
        p.setBuilder(b);
        Director dir = new Director(n, p, parserTable);
        return dir.construct();
    }
}
