/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.workflow;

import kgrid.raems.service.impl.Consts;
import kgrid.raems.service.impl.Utils;
import kgrid.raems.service.impl.utils.XmlUtils;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class PluginDescription implements Consts {

	private String parserName, builderName, parserConstants;

	private static Document pluginDoc;

	static {
		try {
			pluginDoc = XmlUtils.loadFromFile(PLUGIN_PATH);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public PluginDescription(String bName, String pConstants, String pName) {
		this.builderName = bName;
		this.parserConstants = pConstants;
		this.parserName = pName;
	}

	public PluginDescription(String element) throws Exception {
		Node n = pluginDoc.getElementById(element);
		NodeList nodeList = n.getChildNodes();
		for (int i = 0; i < nodeList.getLength(); i++) {
			Node temp = nodeList.item(i);
			if (temp.getNodeName().equals("Parser")) {
				parserName = Utils.cleanString(temp.getTextContent());
			} else if (temp.getNodeName().equals("Builder")) {
				builderName = Utils.cleanString(temp.getTextContent());
			} else if (temp.getNodeName().equals("ParserConstants")) {
				parserConstants = Utils.cleanString(temp.getTextContent());
			}
		}
	}

	public String getParsername() {
		return parserName;
	}

	public String getBuildername() {
		return builderName;
	}

	public String getParserconstants() {
		return parserConstants;
	}

}
