/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.workflow;

import javax.xml.parsers.ParserConfigurationException;

import kgrid.raems.service.impl.utils.ID;
import kgrid.raems.service.impl.workflow.constants.Constants;
import kgrid.raems.service.impl.workflow.constants.JobConstants;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public abstract class Program extends AbstractResource {
    public Program() {
        super();
    }

    public void setId(ID id) {
        super.setId(id);
        if (id.isIDRef()) {
            throw new RuntimeException("A program can't be referenced");
        }
    }

    public abstract ArgumentList getArgs();

    public abstract void setOutputFolder(String outputFolder);

    public abstract String getOutputFolder();

    public String getStandardOutput() {
        return getName() + "_Stdout.txt";
    }

    public String getStandardInput() {
        return "/dev/null";
    }

    public String getStandardError() {
        return getName() + "_StdErr.txt";
    }

    public abstract String getInvocation();

    protected Serializer serializer() throws ParserConfigurationException {
        Serializer ser = new Serializer(JobConstants.Program);
        ser.addAsAttribute(Constants.Href, this.href);
        ser.addAsAttribute(Constants.Title, this.title);
        return ser;
    }
}
