/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.workflow;

import kgrid.raems.service.impl.utils.ID;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * A ReferencedData is a resource which is detailed in another object/xml
 * element; that is, its details are not specified in the current object/xml
 * element but can be found somewhere else. Each resource is detailed only in
 * one object/xml element to avoid redundancy: for all the other occurrencies of
 * the same resource only a reference (idref) to the id of the resource
 * containing the full details is indicated.
 * 
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class ReferencedData extends Data {

	public ReferencedData() {
		super();
	}

	public void setId(ID id) {
		super.setId(id);
		if (id == null || !id.isIDRef()) {
			throw new RuntimeException("A ReferencedData must have an idref!");
		}
	}

	public Resource clone() throws CloneNotSupportedException {
		throw new RuntimeException("A ReferencedData can't be cloned");
	}

	public String getHref() {
		throw new RuntimeException("A ReferencedData doens't have an href");
	}

	public String getName() {
		throw new RuntimeException("A ReferencedData doens't have a name");
	}

	public String getPhysicalLocation() {
		throw new RuntimeException(
				"A ReferencedData doens't have a physical location");
	}

	public String getTitle() {
		throw new RuntimeException("A ReferencedData doens't have a title");
	}

	public boolean isAbstract() {
		return true;
	}

	public boolean isReferenced() {
		return true;
	}

	public void setConcrete(String hostname, String path) {
		throw new RuntimeException(
				"A ReferencedData can't have an hostname or a location");
	}

	public void update(Document metadata) throws Exception {
		throw new RuntimeException("A ReferencedData doens't have an href");
	}

	public String getType() {
		throw new RuntimeException("A ReferencedData doens't have a type");
	}

	public void setType(String type) {
		throw new RuntimeException("A ReferencedData can't set a type");
	}

	public String getHostName() {
		throw new RuntimeException("A ReferencedData can't have an hostname");
	}

	public String toString() {
		return "[ReferencedData:\n" + this.id + "]\n";
	}

	@Override
	public Element serialize() {
		return null;
	}

}
