/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.workflow;

import javax.xml.parsers.ParserConfigurationException;

import kgrid.raems.service.impl.utils.ID;
import kgrid.raems.service.impl.utils.Query;

import org.globus.wsrf.query.QueryException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public interface Resource {

	public String getHostName();

	public String getHref();

	public ID getId();

	public String getName();

	public String getPhysicalLocation();

	public Query getQuery() throws QueryException;

	public String getTitle();

	public boolean isAbstract();

	// Publication is a sufficient condition for concreteness
	public boolean isPublished();

	public boolean isReferenced();

	public void setConcrete(String hostname, String path);

	public void setId(ID id);

	public void setName(String name);

	public void setPublicInformation(String href, String title);

	public void update(Document metadata) throws Exception;

	/**
	 * return a DOM Element representing an XML serialization of the concrete
	 * resource. Fails if the resource is abstract.
	 */
	public Element serialize() throws ParserConfigurationException;

}
