/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.workflow;

import java.security.KeyException;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;

import kgrid.raems.service.impl.utils.ID;
import kgrid.raems.service.impl.workflow.plugin.Dataset;
import kgrid.raems.service.impl.workflow.plugin.DatasetQuerable;

import org.globus.wsrf.NoSuchResourceException;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class ResourceCache {
	private HashMap<Resource, LinkedList<Resource>> resourceReferences = new HashMap<Resource, LinkedList<Resource>>();
	private HashMap<Resource, Resource> referenced = new HashMap<Resource, Resource>();
	private HashMap<ID, Resource> resources = new HashMap<ID, Resource>();
	private LinkedList<Pair<Resource, Task>> owners = new LinkedList<Pair<Resource, Task>>();

	public Resource getResource(ID id) {
		return resources.get(id);
	}

	private void reset() {
		resourceReferences = new HashMap<Resource, LinkedList<Resource>>();
		referenced = new HashMap<Resource, Resource>();
		resources = new HashMap<ID, Resource>();
		owners = new LinkedList<Pair<Resource, Task>>();

	}

	private void addResource(Resource r, Task owner)
			throws NoSuchResourceException {
		if (!r.getId().isIDRef()) {
			resources.put(r.getId(), r);
			resourceReferences.put(r, new LinkedList<Resource>());
			owners.add(new Pair<Resource, Task>(r, owner));
		} else {
			// Resource ref; //mio
			// if(!r.getHref().equals("")) //mio
			// ref = r; //mio
			// else //mio
			// ref = resources.get(r.getId());
			Resource ref = resources.get(r.getId());
			// if (ref == null)//mio
			// ref=r;//mio
			if (ref == null) {
				throw new NoSuchResourceException("No resource with id:"
						+ r.getId());
			}
			LinkedList<Resource> refs = resourceReferences.get(ref);
			refs.add(r);
			referenced.put(r, ref);
		}

	}

	public Collection<Resource> getAll() {
		return resources.values();
	}

	private void removeResource(Resource r) {

	}

	public Resource getReferencedResource(Resource r) throws KeyException {
		if (!r.getId().isIDRef())
			throw new KeyException("No idref for this resource");
		return referenced.get(r);
	}

	public Collection<Resource> getResourceReferences(Resource r)
			throws KeyException {
		if (r.getId().isIDRef())
			throw new KeyException("No id for this resource");
		return resourceReferences.get(r);
	}

	public Task getOwner(Resource r) {
		for (Pair<Resource, Task> pair : owners) {
			if (r.equals(pair.getFirst()))
				return pair.getSecond();
		}
		return null;
	}

	public void buildResourceCache(Job j) throws NoSuchResourceException {
		reset();
		for (Task t : j.getAllTasks()) {
			for (Resource r : t.getResources()) {
				if (!r.getId().isIDRef()) {
					this.addResource(r, t);
				}
			}
		}
		for (Task t : j.getAllTasks()) {
			for (Resource r : t.getResources()) {
				if (r.getId().isIDRef()) {
					this.addResource(r, t);
				}
			}
		}
	}

	public static void main(String[] args) {
		DatasetQuerable d = new DatasetQuerable();
		d.setId(new ID("id", false));
		Dataset d1 = new Dataset();
		d1.setId(new ID("id", true));
		ResourceCache cache = new ResourceCache();
		try {
			cache.addResource(d, null);
			cache.addResource(d1, null);
			System.out.println(cache.getReferencedResource(d1));
		} catch (NoSuchResourceException e) {
			e.printStackTrace();
		} catch (KeyException e) {
			e.printStackTrace();
		}
	}

	public String toString() {
		String s = "";
		for (Resource r : resources.values()) {
			s += r.getId() + "\n";
		}
		return s;
	}
}
