/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.workflow;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class Serializer {
    private Document resource;

    public Serializer(String rootElementTagName)
            throws ParserConfigurationException {
        DocumentBuilderFactory dbfac = DocumentBuilderFactory.newInstance();
        dbfac.setNamespaceAware(false);
        DocumentBuilder docBuilder = dbfac.newDocumentBuilder();
        resource = docBuilder.newDocument();
        resource.appendChild(resource.createElement(rootElementTagName));
    }

    // XXX Anything to do here?
    public Serializer(String rootElementTagName, String prefix, String namespace)
            throws ParserConfigurationException {
        DocumentBuilderFactory dbfac = DocumentBuilderFactory.newInstance();
        DocumentBuilder docBuilder = dbfac.newDocumentBuilder();
        resource = docBuilder.newDocument();
        resource.appendChild(resource.createElement(rootElementTagName));
        resource.getDocumentElement()
                .setAttribute("xmlns:" + prefix, namespace);
    }

    public Element serializedElement() {
        return resource.getDocumentElement();
    }

    public Document serializedDocument() {
        return resource;
    }

    public void addAsAttribute(String name, String value) {
        resource.getDocumentElement().setAttribute(name, value);
    }

    public void addAsChildElement(String name, String content) {
        Element elem = resource.createElement(name);
        elem.setTextContent(content);
        resource.getDocumentElement().appendChild(elem);
    }

    public void addAsChildElement(Element e) {
        Node importedNode = null;
        if (!e.getOwnerDocument().equals(resource))
            importedNode = resource.importNode(e, true);
        else
            importedNode = e;
        resource.getDocumentElement().appendChild(importedNode);
    }
}
