/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.workflow;

import java.io.Serializable;
import java.util.Collection;

import javax.xml.parsers.ParserConfigurationException;

import kgrid.raems.service.impl.workflow.constants.Constants;
import kgrid.raems.service.impl.workflow.constants.JobConstants;

import org.w3c.dom.Element;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public abstract class Task implements Serializable {
    static final long serialVersionUID = -4046455844251785270L;

    private String    name;

    public Task(String name) {
        this.name = name;
    }

    public String getName() {
        return name;
    }

    public boolean equals(Object o) {
        return ((o instanceof Task) && ((Task) o).name.equals(name));
    }

    public int hashCode() {
        return name.hashCode();
    }

    public abstract void addInput(Resource source);

    public abstract void addOutput(Resource source);

    public abstract boolean isAbstract();

    public abstract void replaceResource(Resource from, Resource to);

    public abstract Collection<Resource> getResources();

    public abstract Collection<Resource> getInputs();

    public abstract Collection<Resource> getOutputs();

    public abstract String toString();

    protected Serializer serializer() throws ParserConfigurationException {
        Serializer ser = new Serializer(JobConstants.Task);
        ser.addAsAttribute(Constants.Name, name);
        return ser;
    }

    public abstract Element serialize() throws ParserConfigurationException;
}
