/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.workflow;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedList;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class TaskDAGHashImpl implements TaskDAG {
    static final long                     serialVersionUID = 6544799626616827247L;
    private HashSet<String>               tasks;
    private HashSet<Pair<String, String>> precedences;

    public TaskDAGHashImpl() {
        tasks = new HashSet<String>();
        precedences = new HashSet<Pair<String, String>>();
    }

    public boolean precedes(String taskID1, String taskID2) {
        return precedences.contains(new Pair<String, String>(taskID1, taskID2));
    }

    public void addPrecedence(String taskID1, String taskID2) {
        tasks.add(taskID1);
        tasks.add(taskID2);
        precedences.add(new Pair<String, String>(taskID1, taskID2));
    }

    public void removePrecedence(String taskID1, String taskID2) {
        precedences.remove(new Pair<String, String>(taskID1, taskID2));
    }

    public Collection<String> subsequentTasks(String taskID) {
        ArrayList<String> ret = new ArrayList<String>();
        for (String current_tid : tasks) {
            if (!taskID.equals(current_tid) && precedes(taskID, current_tid))
                ret.add(current_tid);
        }
        return ret;
    }

    public Collection<String> precedingTasks(String taskID) {
        ArrayList<String> ret = new ArrayList<String>();
        for (String currentTaskId : tasks) {
            if (!taskID.equals(currentTaskId) && precedes(currentTaskId, taskID))
                ret.add(currentTaskId);
        }
        return ret;
    }

    public void addTask(String taskID) {
        tasks.add(taskID);
    }

    public void removeTask(String taskID) {
        tasks.remove(taskID);
        for (String currentPrec : precedingTasks(taskID))
            removePrecedence(currentPrec, taskID);
        for (String currentSub : subsequentTasks(taskID))
            removePrecedence(taskID, currentSub);
    }

    public Collection<String> tasks() {
        return new LinkedList<String>(tasks);
    }

    public String toString() {
        return "[DAG [Tasks:" + tasks + "] [Precedences:" + precedences + "] ]";
    }
}
