/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.workflow.parser;

import java.lang.reflect.Method;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Factory class for Job objects. Please refer to Job library documentation for
 * a more accurate description
 * 
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class Director {

    private ParserTable parserTable = null;
    private Parser      parser      = null;
    private Node        root        = null;

    public Director(Node root, Parser parser, ParserTable parserTable)
            throws IllegalArgumentException, IllegalAccessException {

        this.parserTable = parserTable;
        this.parserTable.setDefault();
        this.parser = parser;
        this.root = root;

    }

    private boolean skip(Node node) {
        return (node.getNodeType() == Node.TEXT_NODE)
                || (node.getNodeType() == Node.DOCUMENT_NODE);
    }

    private void invoke(Node node) throws Exception {
        String operation = parserTable
                .getConfigurationValue(node.getNodeName());
        if (operation != null) {
            Method method = parser.getClass().getDeclaredMethod(operation,
                    Node.class);
            method.invoke(parser, node);
        }
    }

    private void visit(Node node) throws Exception {
        if (!skip(node)) {
            invoke(node);
        }
        NodeList nodeList = node.getChildNodes();

        for (int i = 0; i < nodeList.getLength(); i++) {
            visit(nodeList.item(i));
        }

    }

    public Object construct() throws Exception {
        visit(root);
        return parser.endParsing();
    }

}
