/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.workflow.parser;

import java.lang.reflect.Field;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;

import kgrid.raems.service.impl.Utils;
import kgrid.raems.service.impl.workflow.constants.ParserConstants;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class ParserTable {

	private Hashtable<String, String> table = null;
	private static String partialMethodName = "parse";
	private static String defaultValue = "parseElement";
	private Class<? extends ParserConstants> parserConstants = null;

	public ParserTable(Class<? extends ParserConstants> clazz) {

		table = new Hashtable<String, String>();
		this.parserConstants = clazz;

	}

	public void setDefault() throws IllegalArgumentException,
			IllegalAccessException {

		Field[] fields = parserConstants.getFields();
		for (int i = 0; i < fields.length; i++) {
			String s = "";
			s = (String) fields[i].get(s);
			table.put(s, partialMethodName + fields[i].getName());

		}

	}

	public void printConfigurations() {
		Set<String> keys = table.keySet();

		Iterator<String> keyIterator = keys.iterator();
		while (keyIterator.hasNext()) {
			String key = keyIterator.next();
			String value = table.get(key);
			Utils.print("key: " + key + ", methodName: " + value);
		}

	}

	public void putConfiguration(String key, String value) {

		table.put(key, value);
	}

	public String getConfigurationValue(String key) {
		String value = table.get(key);
		if (value != null) {
			return value;
		}
		return defaultValue;
	}

}
