/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.workflow.plugin;

import kgrid.raems.service.impl.exception.BuilderException;
import kgrid.raems.service.impl.utils.ID;
import kgrid.raems.service.impl.workflow.constants.Constants;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class ConcreteDataMiningSoftwareBuilder implements
        DataMiningSoftwareBuilder {
    private DataMiningSoftware result = null;

    public ConcreteDataMiningSoftwareBuilder() {
        result = new DataMiningSoftware();

    }

    public void buildArg(String name, String type, String defaultz,
            boolean mandatory, boolean multivalue, String value) {
        Arg arg = new Arg();
        arg.setName(name);
        arg.addValue(value);
        arg.setType(type);
        arg.setDefault(defaultz);
        arg.setMandatory(mandatory);
        arg.setMultivalue(multivalue);
        result.addArgsProperty(arg);

    }

    public void buildName(String name) {
        result.setName(name);

    }

    public void buildProgram(String href, String title, String id) {
        result.setPublicInformation(href, title);
        if (id != null) {
            ID d_id = new ID(id, false);
            result.setId(d_id);
        } else
            throw new BuilderException("Each Program must exibit an id");

    }

    public void builtElement(String currentElement, String nodeName,
            String textContent) {
        if (currentElement.equals(Constants.Description)) {
            UniqueProperty property = new UniqueProperty();
            property.setName(nodeName);
            property.setValue(textContent);
            result.addDescriptionProperty(property);
        } else if (currentElement.equals(Constants.Usage)) {
            UniqueProperty property = new UniqueProperty();
            property.setName(nodeName);
            property.setValue(textContent);
            result.addUsageProperty(property);
        } else
            throw new BuilderException(
                    "Element is not under the scope of Description or Usage Element");

    }

    public DataMiningSoftware getResult() {
        String parsedOutputFolder = parseOutputFolder(result.getOutputFolder());
        result.setOutputFolder(parsedOutputFolder);
        return result;
    }

    private String parseOutputFolder(String outputFolder) {
        String parsedOutputFolder = "";
        int output_begin = 0;
        if (outputFolder.startsWith("$")) {
            output_begin = outputFolder.indexOf("/");
            if (output_begin < 0 || output_begin > outputFolder.length())
                output_begin = outputFolder.length();
            String property = outputFolder.substring(1, output_begin);
            parsedOutputFolder += System.getenv(property);
        }
        parsedOutputFolder += outputFolder.substring(output_begin);
        return parsedOutputFolder;
    }

    public void buildArgs() {
    }

}
