/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.workflow.plugin;

import java.util.HashSet;

import kgrid.raems.service.impl.exception.BuilderException;
import kgrid.raems.service.impl.utils.ID;
import kgrid.raems.service.impl.workflow.Attribute;
import kgrid.raems.service.impl.workflow.constants.Constants;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class ConcreteDatasetBuilder implements DatasetBuilder {

    private Dataset   dataset          = null;
    private Attribute currentAttribute = null;

    public Dataset getResult() {
        if (dataset == null) {
            throw new BuilderException("");
        }
        return dataset;
    }

    public void buildDataset(String name) {
        if (dataset == null) {
            dataset = new Dataset();
        }
        dataset.setName(name);
    }

    public void buildType(String type) {
        if (dataset == null) {
            throw new BuilderException("Dataset is null");
        }
        dataset.setType(type);

    }

    public void builtElement(String currentElement, String nodeName,
            String textContent) {
        if (currentElement.equals(Constants.Access)) {
            UniqueProperty property = new UniqueProperty();
            property.setName(nodeName);
            property.setValue(textContent);
            dataset.addAccessProperty(property);
        } else if (currentElement.equals(Constants.Format)) {
            UniqueProperty property = new UniqueProperty();
            property.setName(nodeName);
            property.setValue(textContent);
            dataset.addFormatProperty(property);
        } else if (currentElement.equals(Constants.Attribute)) {
            if (currentAttribute != null) {
                dataset.addAttribute(currentAttribute);
                RepeatableProperty property = new RepeatableProperty();
                property.setName(nodeName);
                property.setValue(textContent);
                currentAttribute.addProperty(property);
            }
        } else {
            throw new BuilderException("Can't parse element. currentElement = "
                    + currentElement + ", nodeName = " + nodeName
                    + ", textContent = " + textContent);
        }
    }

    public void buildAttribute(String name, String type) {
        if (currentAttribute != null) {
            dataset.addAttribute(currentAttribute);

            currentAttribute = null;
        } else {
            Attribute attribute = new Attribute();
            attribute.setName(name);
            attribute.setType(type);
            currentAttribute = attribute;
        }
    }

    public void buildInput(String href, String title, String id, String idref) {
        ID dID = null;
        if (id != null && idref == null) {
            dID = new ID(id, false);
        } else if (id == null && idref != null) {
            dID = new ID(idref, true);
        } else {
            throw new BuilderException(
                    "Each Dataset must exibit an id or and idref");
        }
        dataset = new DatasetQuerable();
        dataset.setId(dID);
        dataset.setPublicInformation(href, title);
    }

    public void buildAttributes() {
        if (dataset == null) {
            throw new BuilderException("Dataset is null");
        }
        HashSet<Attribute> attributes = new HashSet<Attribute>();
        dataset.setAttributes(attributes);
    }

    public void buildStructure() {
        if (dataset == null) {
            throw new BuilderException("Dataset is null");
        }
        HashSet<UniqueProperty> format = new HashSet<UniqueProperty>();
        dataset.setFormat(format);

    }

    public void buildOutput(String id) {
        ID d_id = null;
        if (id != null) {
            d_id = new ID(id, false);
        } else {
            throw new BuilderException(
                    "Each Dataset in output must exibit an id");
        }
        dataset = new Dataset();
        dataset.setId(d_id);
    }
}
