/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.workflow.plugin;

import kgrid.raems.service.impl.exception.BuilderException;
import kgrid.raems.service.impl.utils.ID;
import kgrid.raems.service.impl.workflow.Computation;
import kgrid.raems.service.impl.workflow.DataTransfer;
import kgrid.raems.service.impl.workflow.DummyTask;
import kgrid.raems.service.impl.workflow.Job;
import kgrid.raems.service.impl.workflow.Program;
import kgrid.raems.service.impl.workflow.ReferencedData;
import kgrid.raems.service.impl.workflow.Resource;
import kgrid.raems.service.impl.workflow.SchedulerFactory;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class ConcreteJobBuilder implements JobBuilder {
    private Job          result;
    private String       lastID           = null;
    private Computation  lastComputation  = null;
    private DataTransfer lastDataTransfer = null;

    public void buildLink(String from, String to) {
        if (result == null) {
            throw new BuilderException("The Job is null");
        }
        result.link(from, to);

    }

    public void buildAbstractTask(String id) {
        if (result == null) {
            throw new BuilderException("The Job is null");
        }
        addLastTask();
        DummyTask task = new DummyTask(id);
        result.addTask(task);

    }

    public void buildComputation() {
        if (result == null) {
            throw new BuilderException("The Job is null");
        }
        addLastTask();
        if (lastID == null) {
            throw new BuilderException("missing id");
        }
        Computation computation = new Computation(lastID);
        lastComputation = computation;
        lastID = null;
        lastDataTransfer = null;

    }

    public void buildConcreteTask(String id) {
        this.lastID = id;
    }

    private void addLastTask() {

        if (lastDataTransfer != null) {
            result.addTask(lastDataTransfer);
        }
        if (lastComputation != null) {
            result.addTask(lastComputation);
        }
    }

    public void buildDataTransfer(String source, String destination) {
        if (result == null) {
            throw new BuilderException("The Job is null");
        }
        addLastTask();
        if (lastID == null) {
            throw new BuilderException("missing id");
        }
        DataTransfer dataTransfer = new DataTransfer(lastID);
        dataTransfer.setSource(source);
        dataTransfer.setDestination(destination);

        lastDataTransfer = dataTransfer;

        lastID = null;
        lastComputation = null;

    }

    public void buildInputOfTask(Resource i) {
        if (lastComputation == null && lastDataTransfer == null) {
            throw new BuilderException("The task is null");
        }
        if (lastComputation != null) {
            lastComputation.addInput(i);
        } else if (lastDataTransfer != null) {
            lastDataTransfer.addInput(i);
        }
    }

    public void buildProgramOfComputation(Program p) {
        if (lastComputation == null) {
            throw new BuilderException("The computation is null");
        }
        lastComputation.setProgram(p);
    }

    public Job getResult() {
        return result;
    }

    public void buildJob() {
        result = new Job(SchedulerFactory.HASH_DAG);

    }

    public void buildOutputOfComputation(Resource o) {
        if (lastComputation == null) {
            throw new BuilderException("The computation is null");
        }
        lastComputation.addOutput(o);

    }

    public void buildInputOfTask(String idref) {
        ID id = new ID(idref, true);
        ReferencedData data = new ReferencedData();
        data.setId(id);
        buildInputOfTask(data);
    }

}
