/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.workflow.plugin;

import kgrid.raems.service.impl.exception.ParserException;
import kgrid.raems.service.impl.workflow.constants.Constants;
import kgrid.raems.service.impl.workflow.parser.Builder;
import kgrid.raems.service.impl.workflow.parser.Parser;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class DataMiningSoftwareParser implements Parser {
    private DataMiningSoftwareBuilder builder        = null;
    private String                    currentElement = "";

    public void parseProgram(Node node) {
        NamedNodeMap attributes = node.getAttributes();
        Node href = attributes.getNamedItem(Constants.Href);
        Node title = attributes.getNamedItem(Constants.Title);
        Node id = attributes.getNamedItem(Constants.Id);
        if (href == null || title == null)
            throw new ParserException("Malformed xml");
        if (id == null)
            throw new ParserException("Malformed xml");

        builder.buildProgram(href.getTextContent(), title.getTextContent(), id
                .getTextContent());
    }

    public void parseDataMiningSoftware(Node node) {
        NamedNodeMap attributes = node.getAttributes();
        if (attributes.getLength() != 1)
            throw new ParserException("Malformed xml");
        Node name = attributes.getNamedItem(Constants.Name);
        if (name == null)
            throw new ParserException("Malformed xml");
        builder.buildName(name.getTextContent());

    }

    public void parseDescription(Node node) {
        this.currentElement = Constants.Description;
    }

    public void parseUsage(Node node) {
        this.currentElement = Constants.Usage;
    }

    public void parseArgs(Node node) {
        builder.buildArgs();

    }

    public void parseArg(Node node) {
        NamedNodeMap attributes = node.getAttributes();
        Node name = attributes.getNamedItem(Constants.Name);
        Node type = attributes.getNamedItem(Constants.Type);
        Node defaulz = attributes.getNamedItem(Constants.Default);
        Node mandatory = attributes.getNamedItem(Constants.MANDATORY);
        Node multiValue = attributes.getNamedItem(Constants.Multivalue);

        String typeText = null, defaultzText = null;
        boolean mandatoryValue = false;
        if (type != null)
            typeText = type.getTextContent();
        if (defaulz != null)
            defaultzText = defaulz.getTextContent();
        if (mandatory != null)
            mandatoryValue = Boolean.parseBoolean(mandatory.getTextContent());
        if (name == null)
            throw new ParserException("Malformed Xml");
        if (multiValue != null) {
            boolean multivalueValue = Boolean.parseBoolean(multiValue
                    .getTextContent());
            if (multivalueValue)
                builder.buildArg(name.getTextContent(), typeText, defaultzText,
                        mandatoryValue, multivalueValue, node.getTextContent());
        } else {
            builder.buildArg(name.getTextContent(), typeText, defaultzText,
                    mandatoryValue, false, node.getTextContent());
        }
    }

    public void parseElement(Node node) {
        builder.builtElement(currentElement, node.getNodeName(), node
                .getTextContent());
    }

    public Object endParsing() {
        return builder.getResult();
    }

    public void setBuilder(Builder b) {
        this.builder = (DataMiningSoftwareBuilder) b;
    }

}
