/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.workflow.plugin;

import java.lang.reflect.InvocationTargetException;
import java.util.LinkedList;

import kgrid.raems.service.impl.Consts;
import kgrid.raems.service.impl.utils.Query;
import kgrid.raems.service.impl.workflow.Property;
import kgrid.raems.service.impl.workflow.QueryConstructor;
import kgrid.raems.service.impl.workflow.Resource;
import kgrid.raems.service.impl.workflow.constants.Constants;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class DataMiningSoftwareQueryConstructor extends QueryConstructor
		implements Consts {

	public DataMiningSoftwareQueryConstructor(Resource querable) {
		super(querable);
	}

	public Query getQuery() throws IllegalArgumentException, SecurityException,
			IllegalAccessException, InvocationTargetException,
			NoSuchMethodException {
		if (!resource.isAbstract()) {
			return new Query();
		}
		LinkedList<String> constraints = new LinkedList<String>();
		String dmsq = getDataMiningSoftwareQuery();
		String dq = getDescriptionQuery();
		String uq = getUsageQuery();
		LinkedList<String> aq = getArgumentsQuery();

		if (dmsq != null && !dmsq.equals(""))
			constraints.add(dmsq);
		if (dq != null && !dq.equals(""))
			constraints.add(dq);
		if (uq != null && !uq.equals(""))
			constraints.add(uq);
		if (aq != null && aq.size() > 0)
			constraints.addAll(aq);
		return new Query(constraints.toArray(new String[0]));
	}

	private LinkedList<String> getArgumentsQuery() {
		DataMiningSoftware program = (DataMiningSoftware) resource;
		LinkedList<String> query = new LinkedList<String>();
		String constraint;
		if (program.getArgs().size() > 0) {
			for (Arg arg : program.getArgs()) {
				if (arg.getName() != null && !arg.getName().equals("")) {
					constraint = "Arg[" + Constants.Name + "=" + arg.getName()
							+ "]";
					query.add(constraint);
				}
			}
		}
		return query;
	}

	private String getUsageQuery() {
		DataMiningSoftware program = (DataMiningSoftware) resource;
		String query = null;
		if (program.getUsage().size() > 0) {
			query = "Usage{";
			int numProperties = 0;
			for (Property p : program.getUsage())
				if (p.getValue() != null && !p.getValue().equals("")) {
					query += p.getName() + "=" + p.getValue() + ",";
					numProperties++;
				}
			if (numProperties == 0) {
				return null;
			}
			// rimuove l'ultima virgola
			query = query.substring(0, query.length() - 1);
			query += "}";
		}
		return query;
	}

	private String getDescriptionQuery() {
		DataMiningSoftware program = (DataMiningSoftware) resource;
		String query = null;
		int numProperties = 0;
		if (program.getDescription().size() > 0) {
			query = "Description{";
			for (Property p : program.getDescription()) {
				if (p.getValue() != null && !p.getValue().equals("")) {
					query += p.getName() + "=" + p.getValue() + ",";
					numProperties++;
				}
			}
			if (numProperties == 0) {
				return null;
			}
			// rimuove l'ultima virgola
			query = query.substring(0, query.length() - 1);
			query += "}";
		}
		return query;
	}

	private String getDataMiningSoftwareQuery()
			throws IllegalArgumentException, SecurityException,
			IllegalAccessException, InvocationTargetException,
			NoSuchMethodException {
		DataMiningSoftware program = (DataMiningSoftware) resource;
		if (program.getName() != null && !program.getName().equals("")) {
			return "DataMiningSoftware[name=" + program.getName() + "]";
		}
		return null;
	}

}
