/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.workflow.plugin;

import kgrid.raems.service.impl.exception.ParserException;
import kgrid.raems.service.impl.workflow.constants.Constants;
import kgrid.raems.service.impl.workflow.parser.Builder;
import kgrid.raems.service.impl.workflow.parser.Parser;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class DatasetParser implements Parser {

	private DatasetBuilder builder = null;

	private String currentElement = "";

	public void parseInput(Node node) {
		NamedNodeMap attributes = node.getAttributes();
		Node href = attributes.getNamedItem(Constants.Href);
		Node title = attributes.getNamedItem(Constants.Title);
		Node id = attributes.getNamedItem(Constants.Id);
		Node idref = attributes.getNamedItem(Constants.IdRef);
		if (href == null || title == null)
			throw new ParserException("Malformed xml");
		if (id == null && idref != null)

			builder.buildInput(href.getTextContent(), title.getTextContent(),
					null, idref.getTextContent());
		else if (id != null && idref == null)
			builder.buildInput(href.getTextContent(), title.getTextContent(),
					id.getTextContent(), null);
		else if (id == null && idref == null)
			throw new ParserException("Malformed xml");

	}

	public void parseOutput(Node node) {
		NamedNodeMap attributes = node.getAttributes();
		Node id = attributes.getNamedItem(Constants.Id);
		if (id == null)
			throw new ParserException(
					"Output Declaration must have an attribute named id");
		builder.buildOutput(id.getTextContent());
	}

	public void parseDataset(Node node) {
		NamedNodeMap attributes = node.getAttributes();
		Node name = attributes.getNamedItem(Constants.Name);
		if (name != null)
			builder.buildDataset(name.getTextContent());
		Node type = attributes.getNamedItem(Constants.Type);
		if (type != null)
			builder.buildType(type.getTextContent());
	}

	public void parseAccess(Node node) {
		this.currentElement = Constants.Access;
	}

	public void parseStructure(Node node) {
		builder.buildStructure();
	}

	public void parseFormat(Node node) {
		this.currentElement = Constants.Format;
	}

	public void parseAttribute(Node node) {
		this.currentElement = Constants.Attribute;
		NamedNodeMap attributes = node.getAttributes();
		if (attributes.getLength() != 2)
			throw new ParserException("Malformed xml");
		Node name = attributes.getNamedItem(Constants.Name);
		Node type = attributes.getNamedItem(Constants.Type);
		if (name == null || type == null)
			throw new ParserException("Malformed xml");
		builder.buildAttribute(name.getTextContent(), type.getTextContent());
	}

	public void parseAttributes(Node node) {
		builder.buildAttributes();
	}

	public Object endParsing() {
		return builder.getResult();
	}

	public void parseElement(Node node) {
		builder.builtElement(currentElement, node.getNodeName(),
				node.getTextContent());
	}

	public void setBuilder(Builder b) {
		this.builder = (DatasetBuilder) b;
	}

}
