/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.raems.service.impl.workflow.plugin;

import java.lang.reflect.InvocationTargetException;
import java.util.LinkedList;

import kgrid.raems.service.impl.utils.Query;
import kgrid.raems.service.impl.workflow.Attribute;
import kgrid.raems.service.impl.workflow.Property;
import kgrid.raems.service.impl.workflow.QueryConstructor;
import kgrid.raems.service.impl.workflow.Resource;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class DatasetQueryConstructor extends QueryConstructor {

    public DatasetQueryConstructor(Resource querable) {
        super(querable);
    }

    public Query getQuery() throws IllegalArgumentException, SecurityException,
            IllegalAccessException, InvocationTargetException,
            NoSuchMethodException {
        if (!resource.isAbstract()) {
            return new Query();
        }
        LinkedList<String> constraints = new LinkedList<String>();
        String dataset_query = this.getDatasetQuery();
        String access_query = this.getAccessQuery();
        String format_query = this.getFormatQuery();
        LinkedList<String> attributes_query = this.getAttributesQuery();
        if (dataset_query != null)
            constraints.addLast(dataset_query);
        if (access_query != null)
            constraints.addLast(access_query);
        if (format_query != null)
            constraints.addLast(format_query);
        if (attributes_query.size() > 0) {
            constraints.addAll(attributes_query);
        }
        return new Query(constraints.toArray(new String[0]));
    }

    private String getAccessQuery() {
        Dataset input = (Dataset) resource;
        String query = null;
        String currQuery = "Access{";
        for (Property p : input.getAccess()) {
            if (p.getValue() != null && !p.getValue().equals(""))
                currQuery += p.getName() + "=" + p.getValue() + ",";
        }
        if (!currQuery.equals("Access{")) {
            // rimuove l'ultima virgola
            query = currQuery.substring(0, currQuery.length() - 1);
            query += "}";
        }
        return query;
    }

    private String getDatasetQuery() throws IllegalArgumentException,
            SecurityException, IllegalAccessException,
            InvocationTargetException, NoSuchMethodException {
        Dataset input = (Dataset) resource;
        String query = null;
        // String curr_query = "Dataset[";
        // Field[] fields = Dataset.class.getDeclaredFields();
        //
        // for (Field f : fields) {
        // if (f.getType().equals(String.class)) {
        // String accessor_m = "get"
        // + f.getName().substring(0, 1).toUpperCase()
        // + f.getName().substring(1);
        // String value = (String) (Dataset.class.getMethod(accessor_m,
        // (Class[]) null).invoke(input, (Object[]) null));
        // if (value != null && !value.equals(""))
        // curr_query += f.getName() + "=" + value + ",";
        // }
        // }
        // if (!curr_query.equals("Dataset[")) {
        // query = curr_query.substring(0, curr_query.length() - 1);// rimuove
        // // l
        // // 'ultima
        // // virgola
        // query += "]";
        // }
        if (input.getName() != null && !input.getName().equals(""))
            query = "Dataset[name=" + input.getName() + "]";
        return query;
    }

    private String getFormatQuery() {
        Dataset input = (Dataset) resource;
        String query = null;
        String curr_query = "Format{";
        for (Property p : input.getFormat()) {
            if (p.getValue() != null && !p.getValue().equals(""))
                curr_query += p.getName() + "=" + p.getValue() + ",";
        }
        if (!curr_query.equals("Format{")) {
            query = curr_query.substring(0, curr_query.length() - 1);// rimuove
            // l
            // 'ultima
            // virgola
            query += "}";
        }
        return query;
    }

    private LinkedList<String> getAttributesQuery() {
        LinkedList<String> query = new LinkedList<String>();
        Dataset input = (Dataset) resource;
        String attr_query = null;
        for (Attribute a : input.getAttributes()) {
            String curr_query = "Attribute[";
            if (a.getName() != null && !a.getName().equals(""))
                curr_query += "name=" + a.getName() + ",";
            curr_query += "type=" + a.getType();
            if (!curr_query.equals("Attribute[")) {
                attr_query = curr_query += "]";
            } else
                attr_query = curr_query.substring(0, curr_query.length() - 1);// rimuove
            // la
            // quadra
            // aperta
            curr_query = "{";
            for (Property p : a.getProperties()) {
                if (p.getValue() != null && !p.getValue().equals(""))
                    curr_query += p.getName() + "=" + p.getValue() + ",";
            }
            if (!curr_query.equals("{")) {
                attr_query += curr_query.substring(0, curr_query.length() - 1);// rimuove
                // l
                // 'ultima
                // virgola
                attr_query += "}";

            }
            if (!attr_query.equals("Attribute"))
                query.addLast(attr_query);
        }

        return query;
    }

}
